'''
Datalink / Network / Transport layer headers
'''
import string
import socket
import struct

import vstruct
from vstruct.primitives import *

ETH_P_IP    = 0x0800
ETH_P_ARP   = 0x0806
ETH_P_IPv6  = 0x86dd
ETH_P_VLAN  = 0x8100

IPPROTO_ICMP    = 1
IPPROTO_TCP     = 6
IPPROTO_UDP     = 17
IPPROTO_IPV6    = 41
IPPROTO_GRE     = 47
IPPROTO_ICMP6   = 58

TCP_F_FIN  = 0x01
TCP_F_SYN  = 0x02
TCP_F_RST  = 0x04
TCP_F_PUSH = 0x08
TCP_F_ACK  = 0x10
TCP_F_URG  = 0x20
TCP_F_ECE  = 0x40
TCP_F_CWR  = 0x80

# Useful combinations...
TCP_F_SYNACK = (TCP_F_SYN | TCP_F_ACK)

ICMP_ECHOREPLY        =  0
ICMP_DEST_UNREACH     =  3
ICMP_SOURCE_QUENCH    =  4
ICMP_REDIRECT         =  5
ICMP_ECHO             =  8
ICMP_TIME_EXCEEDED    = 11
ICMP_PARAMETERPROB    = 12
ICMP_TIMESTAMP        = 13
ICMP_TIMESTAMPREPLY   = 14
ICMP_INFO_REQUEST     = 15
ICMP_INFO_REPLY       = 16
ICMP_ADDRESS          = 17
ICMP_ADDRESSREPLY     = 18


GREPROTO_PPTP = 0x880b

def reprIPv4Addr(addr):
    bytes = struct.pack('>I', addr)
    return socket.inet_ntoa(bytes)

def decIPv4Addr(addrstr):
    bytes = socket.inet_aton(addrstr)
    return struct.unpack('>I', bytes)[0]

def reprIPv6Addr(addr):
    return socket.inet_ntop(socket.AF_INET6, addr)

class IPv4Address(v_uint32):

    def __init__(self, value=0):
        v_uint32.__init__(self, value=value, bigend=True)

    def __repr__(self):
        bytes = struct.pack('>I', self._vs_value)
        return socket.inet_ntop(socket.AF_INET, bytes)

class IPv6Address(v_bytes):

    def __init__(self, value=0):
        v_bytes.__init__(self, size=16)

    def __repr__(self):
        return socket.inet_ntop(socket.AF_INET6, self._vs_value)

    def vsSetValue(self, val):
        if all(c in string.printable for c in val):
            val = socket.inet_pton(socket.AF_INET6, val)
        if len(val) != self._vs_length:
            raise Exception('v_bytes field set to wrong length!')
        self._vs_value = val

class ETHERII(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.destmac    = v_bytes(size=6)
        self.srcmac     = v_bytes(size=6)
        self.etype      = v_uint16(bigend=True)

    def vsParse(self, sbytes, offset=0, fast=False):
        if fast:
            return vstruct.VStruct.vsParse(self, sbytes, offset=offset, fast=fast)
        # If we end up with a vlan tag, reparse
        ret = vstruct.VStruct.vsParse(self, sbytes, offset=offset)
        if self.etype == ETH_P_VLAN:
            self.vsInsertField('vtag', v_uint16(bigend=True), 'etype')
            self.vsInsertField('vlan', v_uint16(bigend=True), 'etype')
            ret = vstruct.VStruct.vsParse(self, sbytes, offset=offset)
        return ret

class ETHERIIVLAN(ETHERII):
    def __init__(self):
        ETHERII.__init__(self)
        self.vsInsertField('vtag', v_uint16(bigend=True), 'etype')
        self.vsInsertField('vlan', v_uint16(bigend=True), 'etype')

class IPv4(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.veriphl    = v_uint8()
        self.tos        = v_uint8()
        self.totlen     = v_uint16(bigend=True)
        self.ipid       = v_uint16(bigend=True)
        self.flagfrag   = v_uint16(bigend=True)
        self.ttl        = v_uint8()
        self.proto      = v_uint8()
        self.cksum      = v_uint16(bigend=True)
        self.srcaddr    = IPv4Address()
        self.dstaddr    = IPv4Address()

    # Make our len over-ride
    def __len__(self):
        if self.veriphl == 0:
            return vstruct.VStruct.__len__(self)
        return (self.veriphl & 0x0f) * 4

class IPv6(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.verclsflowl= v_uint32(bigend=True)
        self.totlen     = v_uint16(bigend=True)
        self.nexthdr    = v_uint8()
        self.hoplimit   = v_uint8()
        self.srcaddr    = IPv6Address()
        self.dstaddr    = IPv6Address()

class TCP(vstruct.VStruct):

    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.srcport    = v_uint16(bigend=True)
        self.dstport    = v_uint16(bigend=True)
        self.sequence   = v_uint32(bigend=True)
        self.ackseq     = v_uint32(bigend=True)
        self.doff       = v_uint8()
        self.flags      = v_uint8()
        self.window     = v_uint16(bigend=True)
        self.checksum   = v_uint16(bigend=True)
        self.urgent     = v_uint16(bigend=True)

    def __len__(self):
        if self.doff == 0:
            return vstruct.VStruct.__len__(self)
        return self.doff >> 2

class UDP(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.srcport    = v_uint16(bigend=True)
        self.dstport    = v_uint16(bigend=True)
        self.udplen     = v_uint16(bigend=True)
        self.checksum   = v_uint16(bigend=True)

class ICMP(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.type       = v_uint8()
        self.code       = v_uint8()
        self.checksum   = v_uint16(bigend=True)
        #union field starting at offset 4 not included here



import struct

import envi
import envi.common as e_common

import envi.archs.h8.operands as h8_operands

import envi.archs.h8.regs as h8_regs
import envi.archs.h8.parsers as h8_parsers


class H8Disasm:
    fmt = '>H'

    def __init__(self):
        self._dis_regctx = h8_regs.H8RegisterContext()
        self._dis_oparch = envi.ARCH_H8
        self.ptrsize = 4

    def disasm(self, bytez, offset, va):
        """
        Parse a sequence of bytes out into an envi.Opcode instance.
        """
        opval, = struct.unpack_from('>H', bytez, offset)

        prim = opval >> 8
        opdata = h8_parsers.main_table[prim]

        if opdata is None:
            raise envi.InvalidInstruction(bytez=bytez[offset:offset+16], va=va)

        subtable, mnem, decoder, tsize, iflags = opdata

        if subtable:
            raise Exception("NEED subtable at 0x%x:  %s" % (va, e_common.hexify(bytez[offset:offset+16])))

        elif decoder is not None:
            opcode, nmnem, olist, flags, isize = decoder(va, opval, bytez, offset, tsize)
            # print opcode, nmnem, olist, flags, isize, decoder
            if nmnem is not None:
                mnem = nmnem
            iflags |= flags

        else:
            opcode = opval
            isize = 2
            olist = tuple()

        if olist is None:
            raise envi.InvalidInstruction(mesg='Operand list cannot be None for instruction "%s"' % mnem, bytez=bytez[offset:offset+16], va=va)
        op = h8_operands.H8Opcode(va, opcode, mnem, None, isize, olist, iflags)

        if op.opers is not None:
            # following the nasty little hack from other modules.  "everybody's doing it!"
            for oper in op.opers:
                oper._dis_regctx = self._dis_regctx

        return op


if __name__ == '__main__':
    import envi.archs
    envi.archs.dismain(H8Disasm())



import binascii
import unittest

import vivisect
from envi.tests.msp430 import iadc, iadd, iaddc, iand, ibic, ibis, ibit, ibr
from envi.tests.msp430 import icall, iclr, iclrc, iclrn, iclrz, icmp, idadc
from envi.tests.msp430 import idadd, idec, idecd, iinc, iincd, iinv, ijumps
from envi.tests.msp430 import imov, inop, ipop, ipush, iret, irla, irlc, irra
from envi.tests.msp430 import irrc, isbc, isetc, isetn, isetz, isub, isubc
from envi.tests.msp430 import iswpb, isxt, itst, ixor

class msp430InstructionSet(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.longMessage=True

        cls.CODE_VA = 0x4400
        cls.DATA_VA = 0x1000
        cls.MEMSIZE = 0xffff

        cls._vw = vivisect.VivWorkspace()
        cls._vw.setMeta('Architecture', 'msp430')
        cls._vw.setMeta('Platform', 'unknown')
        cls._vw.setMeta('Format', 'blob')
        cls._vw.addMemoryMap(0, 0x7, 'mem', b'\x00' * cls.MEMSIZE)
        cls._emu = cls._vw.getEmulator()

    def doTest(self, test_name, init_state, final_state):
        # Reset memory
        self._emu.writeMemory(0, b"\x00" * self.MEMSIZE)

        # Init registers, status flags and memory
        for reg, val in init_state['regs']:
            self._emu.setRegister(reg, val)

        for flag, state in init_state['flags']:
            self._emu.setFlag(flag, state)

        init_code = binascii.unhexlify(init_state['code'])
        init_data = binascii.unhexlify(init_state['data'])

        self._emu.writeMemory(self.CODE_VA, init_code)
        self._emu.writeMemory(self.DATA_VA, init_data)

        # Emulate instruction
        self._emu.setProgramCounter(self.CODE_VA)
        op = self._vw.arch.archParseOpcode(init_code, 0, self.CODE_VA)
        self._emu.executeOpcode(op)

        # Check final state
        for reg, want in final_state['regs']:
            val = self._emu.getRegister(reg)
            self.assertEqual(val, want, '{} - regs ({})'.format(test_name, reg))

        for flag, want in final_state['flags']:
            val = self._emu.getFlag(flag)
            self.assertEqual(val, want, '{} - flags ({})'.format(test_name, flag))

        want = binascii.unhexlify(final_state['code'])
        code = self._emu.readMemory(self.CODE_VA, len(want))
        self.assertEqual(code, want, test_name + ' - code')

        want = binascii.unhexlify(final_state['data'])
        data = self._emu.readMemory(self.DATA_VA, len(want))
        self.assertEqual(data, want, test_name + ' - data')

    def iterChecks(self, checks):
        for name, init, final in checks:
            self.doTest(name, init, final)

    def test_envi_msp430_adc(self):
        self.iterChecks(iadc.checks)

    def test_envi_msp430_add(self):
        self.iterChecks(iadd.checks)

    def test_envi_msp430_addc(self):
        self.iterChecks(iaddc.checks)

    def test_envi_msp430_and(self):
        self.iterChecks(iand.checks)

    def test_envi_msp430_bic(self):
        self.iterChecks(ibic.checks)

    def test_envi_msp430_bis(self):
        self.iterChecks(ibis.checks)

    def test_envi_msp430_bit(self):
        self.iterChecks(ibit.checks)

    def test_envi_msp430_br(self):
        self.iterChecks(ibr.checks)

    def test_envi_msp430_call(self):
        self.iterChecks(icall.checks)

    def test_envi_msp430_clr(self):
        self.iterChecks(iclr.checks)

    def test_envi_msp430_clrc(self):
        self.iterChecks(iclrc.checks)

    def test_envi_msp430_clrn(self):
        self.iterChecks(iclrn.checks)

    def test_envi_msp430_clrz(self):
        self.iterChecks(iclrz.checks)

    def test_envi_msp430_cmp(self):
        self.iterChecks(icmp.checks)

    def test_envi_msp430_dadc(self):
        self.iterChecks(idadc.checks)

    def test_envi_msp430_dadd(self):
        self.iterChecks(idadd.checks)

    def test_envi_msp430_dec(self):
        self.iterChecks(idec.checks)

    def test_envi_msp430_decd(self):
        self.iterChecks(idecd.checks)

    def test_envi_msp430_dint(self):
        # Not implemented
        pass

    def test_envi_msp430_eint(self):
        # Not implemented
        pass

    def test_envi_msp430_inc(self):
        self.iterChecks(iinc.checks)

    def test_envi_msp430_incd(self):
        self.iterChecks(iincd.checks)

    def test_envi_msp430_inv(self):
        self.iterChecks(iinv.checks)

    def test_envi_msp430_jumps(self):
        self.iterChecks(ijumps.checks)

    def test_envi_msp430_mov(self):
        self.iterChecks(imov.checks)

    def test_envi_msp430_nop(self):
        self.iterChecks(inop.checks)

    def test_envi_msp430_pop(self):
        self.iterChecks(ipop.checks)

    def test_envi_msp430_push(self):
        self.iterChecks(ipush.checks)

    def test_envi_msp430_ret(self):
        self.iterChecks(iret.checks)

    def test_envi_msp430_reti(self):
        # Not implemented
        pass

    def test_envi_msp430_rla(self):
        self.iterChecks(irla.checks)

    def test_envi_msp430_rlc(self):
        self.iterChecks(irlc.checks)

    def test_envi_msp430_rra(self):
        self.iterChecks(irra.checks)

    def test_envi_msp430_rrc(self):
        self.iterChecks(irrc.checks)

    def test_envi_msp430_sbc(self):
        self.iterChecks(isbc.checks)

    def test_envi_msp430_setc(self):
        self.iterChecks(isetc.checks)

    def test_envi_msp430_setn(self):
        self.iterChecks(isetn.checks)

    def test_envi_msp430_setz(self):
        self.iterChecks(isetz.checks)

    def test_envi_msp430_sub(self):
        self.iterChecks(isub.checks)

    def test_envi_msp430_subc(self):
        self.iterChecks(isubc.checks)

    def test_envi_msp430_swpb(self):
        self.iterChecks(iswpb.checks)

    def test_envi_msp430_sxt(self):
        self.iterChecks(isxt.checks)

    def test_envi_msp430_tst(self):
        self.iterChecks(itst.checks)

    def test_envi_msp430_xor(self):
        self.iterChecks(ixor.checks)



'''
One day we might replace this/add to this with a side by side lockstepper with some other emulator,
but for now, this is both faster to make and vastly more self-contained (which is nice)
'''
import binascii
import unittest

import envi
import envi.expression as e_expr

'''
Tuples are (Opcode Bytes, Register Setup, Stack Setup, Postconditions)
Where:
* Opcode Bytes
    * Bytes making up the instrucion
* Register Setup
    * Prepopulate register with specific values before running the instruction tests
      running the emulator
* Stack Setup
    * Any values that should be pushed onto the stack.
      Keyed by an expression that's evaluated by the envi.expression.evaluate method
* Post Registers
    * The assertions on registers memory once the emulator has been run.
      Keyed by register name.
* Post Stack
    * The assertions on registers/stack memory once the emulator has been run.
      Keyed by offset from esp.

Leave Register Setup and/or Stack Setup to take the defaults
'''

DEFREG = 0x00BADB1D
STACKBASE = 0x80

i386Tests = [
    # push 0x12345678
    {'bytes': '6878563412',
     'setup': ({}, {}),
     'tests': ({'esp': STACKBASE-4}, {'esp': b'\x78\x56\x34\x12'})},
    # push fs
    {'bytes': '0fa0',
     'setup': ({'fs': 0xBAD1}, {}),
     'tests': ({}, {'esp': b'\xD1\xBA'})},
    # mov ebx, 47
    {'bytes': 'bb2f000000',
     'setup': ({}, {}),
     'tests': ({'ebx': 47}, {})},
    # bsr ecx, edx
    {'bytes': '0FBDCA',
     'setup': ({'edx': 0x470000, 'eflags': 0x202}, {}),
     'tests': ({'ecx': 22, 'eflags':0x202}, {})},
    # bsr ecx, edx
    {'bytes': '0FBDCA',
     'setup': ({'edx': 0x470000, 'eflags': 0xffffffff}, {}),
     'tests': ({'ecx': 22, 'eflags':0x44602}, {})},     # eflags verified in vdb
    # bsr ecx, edx
    {'bytes': '0FBDC9',
     'setup': ({'ecx': 0, 'eflags':0x202}, {}),
     'tests': ({'ecx': 0, 'eflags':0x246}, {})},        # eflags verified in vdb
    # bsr ecx, edx
    {'bytes': '0FBDC9',
     'setup': ({'ecx': 0, 'eflags':0xffffffff}, {}),
     'tests': ({'ecx': 0, 'eflags':0x44646}, {})},      # eflags verified in vdb
    # push word [esp+2]
    {'bytes': '66ff742402',
     'setup': ({}, {'esp': b'\xCD\xFE\x89\x43'}),
     'tests': ({'esp': STACKBASE-2}, {'esp': b'\x89\x43'})},
    # push [esp]
    {'bytes': 'FF3424',
     'setup': ({}, {'esp': b'\x01\x02\x03\x04'}),
     'tests': ({'esp': STACKBASE-4}, {'esp+4': b'\x01\x02\x03\x04'})},
    # add byte [ecx],al
    {'bytes': '0001',
     'setup': ({'ecx': 0x80, 'eax': 57}, {'esp': b'\x15\x00\x00\x00'}),
     'tests': ({}, {'esp': b'\x4e\x00\x00\x00'})},
    # sub edx, ecx
    {'bytes': '29ca',
     'setup': ({'edx': 0xf9e7cd45, 'ecx': 0xbfcdef00}, {}),
     'tests': ({'edx': 0x3a19de45, 'eflags': 0}, {})},
    # mul edx
    {'bytes': 'f7e2',
     'setup': ({'eax': 0x47b3, 'edx': 0x898937}, {}),
     'tests': ({'eax': 0x85393275, 'edx': 0x26}, {})},
    # div ax
    {'bytes': '66f7f0',
     'setup': ({'eax': 48, 'edx': 1}, {}),
     'tests': ({'eax': 1366, 'edx': 16}, {})},
    # div eax
    {'bytes': 'f7f0',
     'setup': ({'eax': 48, 'edx': 1}, {}),
     'tests': ({'eax': 89478486, 'edx': 16}, {})},
    # idiv edi
    {'bytes': 'f7ff',
     'setup': ({'eax': 0xffffffe2, 'edx': 0xffffffff}, {}),
     'tests': ({'eax': 0, 'edx': 0xffffffe2}, {})},
    # rol eax, 4
    {'bytes': 'c1c004',
     'setup': ({'eax': 5}, {}),
     'tests': ({'eax': 80}, {})},
    # rol eax, 8
    {'bytes': 'c1c008',
     'setup': ({'eax': 0xABCD0000}, {}),
     'tests': ({'eax': 0xCD0000AB, 'eflags': 1}, {})},
    # ror eax, 5
    {'bytes': 'c1c805',
     'setup': ({'eax': 0x79}, {}),
     'tests': ({'eax': 0xc8000003, 'eflags': 1}, {})},
    # ror eax, cl
    {'bytes': 'd3c8',
     'setup': ({'eax': 0x79, 'cl': 1}, {}),
     'tests': ({'eax': 0x8000003c, 'eflags': 0x801}, {})},
    # btr ecx, 17
    {'bytes': '0fbaf111',
     'setup': ({'ecx': 0xF002000F}, {}),
     'tests': ({'eflags': 1, 'ecx': 0xF000000F}, {})},
    # lea ecx,dword [esp + -973695896]  (test SIB getOperAddr)
    {'bytes': '8d8c246894f6c5',
     'setup': ({'esp': 0x7fd0}, {}),
     'tests': ({'ecx': 0xc5f71438}, {})},
    # rep ret
    # The behavior of the REP prefix is undefined when used with non-string instructions.
    {'bytes': 'f3c3',
     'setup': ({'ecx': 0x1}, {'esp': b'\x00\x00\x00\x60'}),
     'tests': ({'ecx': 0x1, 'eip': 0x60000000}, {})}
]


class IntelEmulatorTests(object):
    def setEmuDefaults(self, emu):
        for name in emu.getRegisterNames():
            emu.setRegisterByName(name, DEFREG)
        emu.addMemoryMap(0, 6, "[stack]", b'\xAB' * 0x100)
        emu.setStackCounter(STACKBASE)
        emu.setRegisterByName('eflags', 0)

    def run_emulator_tests(self, arch, tests):
        emu = arch.getEmulator()
        self.setEmuDefaults(emu)
        for test in tests:
            byts = test['bytes']
            try:
                op = arch.archParseOpcode(binascii.unhexlify(byts), 0, 0x40)
            except envi.InvalidInstruction:
                self.fail('Failed to parse opcode bytes: %s' % byts)

            with emu.snap():
                # do any required setup
                for name, valu in test['setup'][0].items():
                    emu.setRegisterByName(name, valu)

                for expr, valu in test['setup'][1].items():
                    addr = e_expr.evaluate(expr, emu.getRegisters())
                    valu = emu.writeMemory(addr, valu)

                # run the emulator instruction
                emu.executeOpcode(op)

                # test both the registers and stack values
                for name, valu in test['tests'][0].items():
                    reg = emu.getRegisterByName(name)
                    self.assertEqual(reg, valu, msg='(reg: %r) Given != Got for %s (%s)' % (name, byts, str(op)))

                for expr, valu in test['tests'][1].items():
                    addr = e_expr.evaluate(expr, emu.getRegisters())
                    mem = emu.readMemory(addr, len(valu))
                    self.assertEqual(mem, valu, msg='(mem: 0x%x) %r != %r' % (addr, mem, valu))

class i386EmulatorTests(unittest.TestCase, IntelEmulatorTests):
    def test_i386_emulator(self):
        arch = envi.getArchModule('i386')
        self.run_emulator_tests(arch, i386Tests)

    def test_x86_OperAddrs(self):
        arch32 = envi.getArchModule('i386')
        emu32 = arch32.getEmulator()
        emu32.setStackCounter(0x7fd0)
        
        op = arch32.archParseOpcode(bytes.fromhex('8d8c246894f6c5'), 0,0)
        self.assertEqual(op.opers[1].getOperAddr(op, emu32), 0xc5f71438)

        import envi.archs.i386.disasm as eaid
        oper = eaid.i386SibOper(tsize=4, reg=eaid.REG_ESP, index=5, scale=2, disp=-0x98765432)
        oper._dis_regctx = eaid.i386RegisterContext()
        addr32 = oper.getOperAddr(op, emu32)
        self.assertEqual(addr32, 0x678a2b9e)

        oper = eaid.i386SibOper(tsize=4, imm=256, index=5, scale=2, disp=-0x98765432)
        oper._dis_regctx = eaid.i386RegisterContext()
        addr32 = oper.getOperAddr(op, emu32)
        self.assertEqual(addr32, 0x6789acce)

        oper = eaid.i386RegMemOper(reg=eaid.REG_ESP, tsize=4, disp=-0x98765432)
        oper._dis_regctx = eaid.i386RegisterContext()
        addr32 = oper.getOperAddr(op, emu32)
        self.assertEqual(addr32, 0x678a2b9e)

    def test_i386_reps(self):
        arch32 = envi.getArchModule('i386')
        emu32 = arch32.getEmulator()
        emu32.addMemoryMap(0x40000000, 7, 'test', b'foobarbazbane' + b'\0'*100)
        op_movsd = arch32.archParseOpcode(bytes.fromhex('f3a5'), 0, 0)  # rep movsd
        op_movsb = arch32.archParseOpcode(bytes.fromhex('f3a4'), 0, 0)  # rep movsb
        op_scasb = arch32.archParseOpcode(bytes.fromhex('f2ae'), 0, 0)  # repnz scasb
        op_zscasb = arch32.archParseOpcode(bytes.fromhex('f3ae'), 0, 0)  # repz scasb

        # rep Dword copy
        desiredoutput1 = b'foobarbazbane\0\0\0foobarbazbane\0\0\0'
        emu32.setRegisterByName('ecx', 5)
        emu32.setRegisterByName('esi', 0x40000000)
        emu32.setRegisterByName('edi', 0x40000010)
        emu32.executeOpcode(op_movsd)
        self.assertEqual(emu32.readMemory(0x40000000, 0x20), desiredoutput1)

        # rep Byte copy
        desiredoutput2 = b'foobarbazbane\0\0\0foobarbazbane\0\0\0foobarbazbane\0\0\0'
        emu32.setRegisterByName('ecx', 16)
        emu32.setRegisterByName('esi', 0x40000000)
        emu32.setRegisterByName('edi', 0x40000020)
        emu32.executeOpcode(op_movsb)
        self.assertEqual(emu32.readMemory(0x40000000, 0x30), desiredoutput2)

        # repnz Byte scan string
        emu32.setRegisterByName('eax', 0)
        emu32.setRegisterByName('ecx', -1)
        emu32.setRegisterByName('edi', 0x40000000)
        emu32.executeOpcode(op_scasb)
        self.assertEqual(emu32.getRegisterByName('ecx'), 0xffffffff - len("foobarbazbane\x00"))

        # repz Byte scan string
        emu32.setRegisterByName('eax', 0)
        emu32.setRegisterByName('ecx', -1)
        emu32.setRegisterByName('edi', 0x4000000d)
        emu32.executeOpcode(op_zscasb)
        self.assertEqual(emu32.getRegisterByName('ecx'), 0xffffffff - len("\x00\x00\x00f"))

        # repnz Byte scan string with repmax
        TEST_REPMAX = 2
        orig_repmax = emu32.getEmuOpt("i386:repmax")
        emu32.setEmuOpt("i386:repmax", TEST_REPMAX)

        emu32.setRegisterByName('eax', 0)
        emu32.setRegisterByName('ecx', -1)
        emu32.setRegisterByName('edi', 0x40000000)
        emu32.executeOpcode(op_scasb)
        self.assertEqual(emu32.getRegisterByName('ecx'), 0xffffffff - min(TEST_REPMAX, len("foobarbazbane\x00")))

        emu32.setEmuOpt("i386:repmax", orig_repmax)

        # repz Byte scan string with repmax
        TEST_REPMAX = 2
        orig_repmax = emu32.getEmuOpt("i386:repmax")
        emu32.setEmuOpt("i386:repmax", TEST_REPMAX)

        emu32.setRegisterByName('eax', 0)
        emu32.setRegisterByName('ecx', -1)
        emu32.setRegisterByName('edi', 0x4000000d)
        emu32.executeOpcode(op_zscasb)
        self.assertEqual(emu32.getRegisterByName('ecx'), 0xffffffff - min(TEST_REPMAX, len("\x00\x00\x00f")))

        emu32.setEmuOpt("i386:repmax", orig_repmax)



import unittest

import vivisect.reports as v_reports
import vivisect.tests.helpers as helpers


class ReportsTest(unittest.TestCase):
    '''
    Test each of the base report modules.
    '''
    @classmethod
    def setUpClass(cls):
        cls.vw = helpers.getTestWorkspace('windows', 'i386', 'helloworld.exe')

    def test_overlap(self):
        cols, retn = v_reports.runReportModule(self.vw, 'vivisect.reports.overlaplocs')
        self.assertEqual(cols, (("Overlap Size",   int),
                                ("This Location",  str),
                                ("Other Location", str)))

        self.assertGreater(len(retn), 0)
        for va, meta in retn.items():
            size, baserepr, othrrepr = meta
            self.assertGreater(size, 0)
            self.assertNotEqual(baserepr, '')
            self.assertNotEqual(othrrepr, '')

    def test_undef(self):
        cols, retn = v_reports.runReportModule(self.vw, 'vivisect.reports.undeftargets')
        self.assertEqual(cols, (("Bytes", str),
                                ("Name", str)))

        self.assertGreater(len(retn), 0)
        for va, undef in retn.items():
            byts, mesg = undef
            self.assertIsNone(self.vw.getLocation(va))
            self.assertGreater(len(byts), 0)
            self.assertGreater(len(mesg), 0)

    def test_locationdist(self):
        cols, retn = v_reports.runReportModule(self.vw, 'vivisect.reports.locationdist')
        self.assertEqual(cols, (("Location Type", str),
                                ("Instance Count", int),
                                ("Size (bytes)", int),
                                ("Size (percent)", int)))

        self.assertEqual(retn, self.vw.getLocationDistribution())

    def test_funcomp(self):
        cols, retn = v_reports.runReportModule(self.vw, 'vivisect.reports.funccomplexity')
        self.assertEqual(cols, (("Code Blocks", int),
                                ("Mnem Dist", int)))
        vw = self.vw

        self.assertGreater(len(retn), 0)
        for fva, comp in retn.items():
            blks, mdist = comp
            self.assertEqual(blks, len(vw.getFunctionBlocks(fva)))
            self.assertEqual(mdist, vw.getFunctionMeta(fva, 'MnemDist', -1))



import logging

logger = logging.getLogger(__name__)

# Heap Flags
HEAP_NO_SERIALIZE               = 0x00000001
HEAP_GROWABLE                   = 0x00000002
HEAP_GENERATE_EXCEPTIONS        = 0x00000004
HEAP_ZERO_MEMORY                = 0x00000008
HEAP_REALLOC_IN_PLACE_ONLY      = 0x00000010
HEAP_TAIL_CHECKING_ENABLED      = 0x00000020
HEAP_FREE_CHECKING_ENABLED      = 0x00000040
HEAP_DISABLE_COALESCE_ON_FREE   = 0x00000080
HEAP_CREATE_ALIGN_16            = 0x00010000
HEAP_CREATE_ENABLE_TRACING      = 0x00020000
HEAP_CREATE_ENABLE_EXECUTE      = 0x00040000

heap_flag_names = {
HEAP_NO_SERIALIZE:"HEAP_NO_SERIALIZE",
HEAP_GROWABLE:"HEAP_GROWABLE",
HEAP_GENERATE_EXCEPTIONS:"HEAP_GENERATE_EXCEPTIONS",
HEAP_ZERO_MEMORY:"HEAP_ZERO_MEMORY",
HEAP_REALLOC_IN_PLACE_ONLY:"HEAP_REALLOC_IN_PLACE_ONLY",
HEAP_TAIL_CHECKING_ENABLED:"HEAP_TAIL_CHECKING_ENABLED",
HEAP_FREE_CHECKING_ENABLED:"HEAP_FREE_CHECKING_ENABLED",
HEAP_DISABLE_COALESCE_ON_FREE:"HEAP_DISABLE_COALESCE_ON_FREE",
HEAP_CREATE_ALIGN_16:"HEAP_CREATE_ALIGN_16",
HEAP_CREATE_ENABLE_TRACING:"HEAP_CREATE_ENABLE_TRACING",
HEAP_CREATE_ENABLE_EXECUTE:"HEAP_CREATE_ENABLE_EXECUTE",
}

# Heap Chunk Flags
HEAP_ENTRY_BUSY             = 0x01
HEAP_ENTRY_EXTRA_PRESENT    = 0x02
HEAP_ENTRY_FILL_PATTERN     = 0x04
HEAP_ENTRY_VIRTUAL_ALLOC    = 0x08
HEAP_ENTRY_LAST_ENTRY       = 0x10
HEAP_ENTRY_SETTABLE_FLAG1   = 0x20
HEAP_ENTRY_SETTABLE_FLAG2   = 0x40
HEAP_ENTRY_SETTABLE_FLAG3   = 0x80



def reprHeapFlags(flags):
    ret = []
    if flags & HEAP_ENTRY_BUSY:
        ret.append("BUSY")
    if flags & HEAP_ENTRY_FILL_PATTERN:
        ret.append("FILL")
    if flags & HEAP_ENTRY_LAST_ENTRY:
        ret.append("LAST")
    if len(ret):
        return "|".join(ret)
    return "NONE"

class HeapCorruptionException(Exception):
    def __init__(self, heap, segment, prevchunk, badchunk):
        prevaddr = 0
        badaddr = 0
        if prevchunk is not None:
            prevaddr = prevchunk.address
        if badchunk is not None:
            badaddr = badchunk.address
        Exception.__init__(self, "Prev Chunk: 0x%.8x Bad Chunk: 0x%.8x" % (prevaddr, badaddr))
        self.heap = heap
        self.segment = segment
        self.prevchunk = prevchunk
        self.badchunk = badchunk

class FreeListCorruption(Exception):
    def __init__(self, heap, index, prevchunk, badchunk):
        prevaddr = 0
        badaddr = 0
        if prevchunk is not None:
            prevaddr = prevchunk.address
        if badchunk is not None:
            badaddr = badchunk.address
        Exception.__init__(self, "Index: %d Prev Chunk: 0x%.8x Bad Chunk: 0x%.8x" % (index, prevaddr, badaddr))
        self.heap = heap
        self.index = index
        self.prevchunk = prevchunk
        self.badchunk = badchunk

class ChunkNotFound(Exception):
    pass

def getHeapSegChunk(trace, address):
    """
    Find and return the heap, segment, and chunk for the given addres
    (or exception).
    """
    for heap in getHeaps(trace):

        for seg in heap.getSegments():

            segstart = seg.address
            segend = seg.getSegmentEnd()

            if address < segstart or address > segend:
                continue

            for chunk in seg.getChunks():
                a = chunk.address
                b = chunk.address + len(chunk)
                if address >= a and address < b:
                    return heap,seg,chunk

    raise ChunkNotFound("No Chunk Found for 0x%.8x" % address)

def getHeaps(trace):
    """
    Get the win32 heaps (returns a list of Win32Heap objects)
    """
    ret = []
    pebaddr = trace.getMeta("PEB")
    peb = trace.getStruct("ntdll.PEB", pebaddr)
    heapcount = int(peb.NumberOfHeaps)
    hlist = trace.readMemoryFormat(int(peb.ProcessHeaps), "<"+('P'*heapcount))
    for haddr in hlist:
        ret.append(Win32Heap(trace, haddr))
    return ret

class Win32Heap:
    def __init__(self, trace, address):
        self.address = address
        self.trace = trace
        self.heap = trace.getStruct('ntdll.HEAP', address)
        self._win7_heap = False
        self.heapenc = None
        if self.heap.vsHasField('Encoding'):
            self.heapenc = self.heap.Encoding
            self._win7_heap = True
        self.seglist = None
        self.ucrdict = None

    def isLFH(self):
        '''
        Does this heap have LFH enabled?
        '''
        if self.heap.FrontEndHeapType == 0x2:
            return True

        return False

    def getLFH(self):
        '''
        return LFH object
        '''
        if self.isLFH():
            return Win32LFH(self.trace, self.heap, self.heap.FrontEndHeap)

    def hasLookAside(self):
        '''
        Does this heap have a lookaside?
        '''
        if not self.heap.Flags & HEAP_GROWABLE:
            return False
        if self.heap.Flags & HEAP_NO_SERIALIZE:
            return False
        return True

    def getUCRDict(self):
        '''
        Retrieve a dictionary of <ucr_address>:<ucr_size> items.

        (If this windows version doesn't support UCRs, the dict will be empty)
        '''
        if self.ucrdict is None:
            self.ucrdict = {}
            if self.heap.vsHasField('UCRList'):
                listhead_va = self.address + self.heap.vsGetOffset('UCRList')
                for lva in self._getListEntries(listhead_va):
                    ucrent = self.trace.getStruct('ntdll.HEAP_UCR_DESCRIPTOR', lva)
                    if ucrent.Size != 0:
                        self.ucrdict[ucrent.Address] = ucrent.Size

        return self.ucrdict

    def _win7ParseSegments(self):
        # Address doesn't matter for the below call
        heapseg = self.trace.getStruct('ntdll.HEAP_SEGMENT', self.address)

        listhead = self.address + self.heap.vsGetOffset('SegmentList')
        # Negative offset from segment list entry to segment
        segoff = heapseg.vsGetOffset('SegmentListEntry')
        entry = self.heap.SegmentList.Flink
        while entry != listhead:
            seg = Win32Segment(self.trace, self, entry - segoff)
            self.seglist.append(seg)
            entry = self.trace.readMemoryFormat(entry, '<P')[0]

    def getSegments(self):
        '''
        Return a list of Win32Segment objects.
        '''
        if self.seglist is None:
            self.seglist = []

            # Windows 7 style heap segment list
            if self.heap.vsHasField('SegmentList'):
                self._win7ParseSegments()

            else:
                for i in range(int(self.heap.LastSegmentIndex)+1):
                    sa = self.heap.Segments[i]
                    self.seglist.append(Win32Segment(self.trace, self, int(sa)))

        return self.seglist

    def getLookAsideLists(self):
        '''
        Return a list of the lookaside list for this heap
        '''
        if not self.hasLookAside():
            raise Exception('Heap at 0x%.8x has no lookaside!' % (self.address))
        # Look aside lists are 128 pointer slots in a chunk pointed
        # to by the FrontEndHeap field in the heap structure.
        #fetype = self.heap.FrontEndHeapType

        laside = self.heap.FrontEndHeap
        ret = []
        for i in range(128):
            slot = laside + (i * 0x30)
            bucket = []
            base = self.trace.readMemoryFormat(slot, "<I")[0]
            while base != 0:
                chunk = Win32Chunk(self.trace, self, base-8)
                bucket.append(chunk)
                base,blink = chunk.getFlinkBlink()
            ret.append(bucket)
        return ret

    def _getListEntries(self, addr, listhead=True):
        ret = []
        if not listhead:
            ret.append(addr)
        le = self.trace.getStruct('ntdll.LIST_ENTRY', addr)
        while le.Flink != addr:
            ret.append(le.Flink)
            le = self.trace.getStruct('ntdll.LIST_ENTRY', le.Flink)
        return ret

    def _win7FreeLists(self):
        logger.warning('Windows 7 Free List Parsing Fixme!')
        return []

    def getFreeLists(self):
        '''
        Return a list of the free lists in this heap.
        (Not including look-aside)
        '''
        ret = []
        foff = self.heap.vsGetOffset('FreeLists')
        if self._win7_heap:
            return self._win7FreeLists()

        for i in range(128):
            le = self.heap.FreeLists[i]
            bucket = []
            base = self.address + foff + (i*8)

            addr = le.Flink

            while addr != base:

                # If we die here, the guy before us was dorked.
                try:
                    chunk = Win32Chunk(self.trace, self, addr-8)
                except Exception as e:
                    chunk = bucket[-1]
                    pchunk = None
                    if len(bucket) >= 2:
                        pchunk = bucket[-2]
                    raise FreeListCorruption(self, i, pchunk, chunk)

                # Check our back pointer
                flink,blink = chunk.getFlinkBlink()
                if len(bucket):
                    pchunk = bucket[-1]
                    if blink != pchunk.getDataAddress():
                        raise FreeListCorruption(self, i, pchunk, chunk)

                addr = flink
                bucket.append(chunk)

            ret.append(bucket)
        return ret

    def getFlagNames(self):
        ret = []
        for k,v in heap_flag_names.items():
            if self.heap.Flags & k:
                ret.append(v)
        return ret

    def __repr__(self):
        fnames = "|".join(self.getFlagNames())
        return "heap: 0x%.8x flags:%s" % (self.address, fnames)

class Win32Segment:
    def __init__(self, trace, heap, address):
        self.trace = trace
        self.heap = heap
        self.address = address
        self.seg = trace.getStruct('ntdll.HEAP_SEGMENT', address)
        #FIXME segments can specify chunk Size granularity
        self.chunks = None
        self.segend = self.address + (self.seg.NumberOfPages * 4096)

    def getSegmentEnd(self):
        return self.segend

    def getChunks(self):
        if self.chunks is None:
            self.chunks = []
            addr = self.address
            lastchunk = None
            ucrdict = self.heap.getUCRDict()

            while True:

                # Skip any uncommited ranges...
                usize = ucrdict.get(addr)
                if usize is not None:
                    lastchunk = None
                    addr += usize
                    continue

                # Since an un-commited range may put us past the
                # lastblock (segend) we must double check...
                if addr >= self.segend:
                    break

                chunk = Win32Chunk(self.trace, self.heap, addr)

                self.chunks.append(chunk)
                #if lastchunk is not None:
                    #if lastchunk.chunk.Size != chunk.chunk.PreviousSize:
                        #logger.warning('last size: %d, prev:%d' % (lastchunk.chunk.Size,chunk.chunk.PreviousSize))
                        #raise HeapCorruptionException(self.heap, self, lastchunk, chunk)

                if chunk.isLast():
                    break

                lastchunk = chunk
                addr += len(chunk)
        return self.chunks

    def getLastChunk(self):
        va = self.seg.LastEntryInSegment
        return Win32Chunk(self.trace, self.heap, va)

class Win32Chunk:
    def __init__(self, trace, heap, address):
        self.trace = trace
        self.heap = heap
        self.address = address
        self.chunk = trace.getStruct('ntdll.HEAP_ENTRY', address)

        # Decode the heap chunk if needed...
        if self.heap.heapenc:
            self.chunk ^= self.heap.heapenc

    def __repr__(self):
        return 'HeapChunk: 0x%.8x (%d) %s' % (self.address, len(self),self.reprFlags())

    def __len__(self):
        return int(self.chunk.Size) * 8

    def isLast(self):
        return bool(int(self.chunk.Flags) & HEAP_ENTRY_LAST_ENTRY)

    def isBusy(self):
        return bool(int(self.chunk.Flags) & HEAP_ENTRY_BUSY)

    def getDataAddress(self):
        return self.address + len(self.chunk)

    def getDataSize(self):
        return len(self) - len(self.chunk)

    def getDataBytes(self, maxsize=None):
        size = self.getDataSize()
        if maxsize is not None:
            size = min(size, maxsize)
        return self.trace.readMemory(self.getDataAddress(), size)

    def getFlinkBlink(self):
        return self.trace.readMemoryFormat(self.getDataAddress(), '<PP')

    def reprFlags(self):
        return reprHeapFlags(int(self.chunk.Flags))

class Win32Subsegment(object):
    def __init__(self, trace, heap, address):
        self.trace = trace
        self.heap = heap
        self.address = address
        self.subsegment = trace.getStruct('ntdll.HEAP_SUBSEGMENT', address)

    def getChunks(self):
        for chunk in range(self.getChunksStart(), self.getChunksEnd(), self.getBucketSize()):
            yield Win32Chunk(self.trace, self.heap, chunk)

    def getUserBlocks(self):
        return self.subsegment.UserBlocks

    def getChunksStart(self):
        # Length of HEAP_USERDATA_HEADER = 0x10
        return self.getUserBlocks() + 0x10

    def getChunksEnd(self):
        return self.getChunksStart() + (self.subsegment.BlockCount * self.getBucketSize())

    def getBucketSize(self):
        if self.trace.getMeta('Architecture') == 'amd64':
            return self.subsegment.BlockSize * 16
        else:
            return self.subsegment.BlockSize * 8

    def getSizeIndex(self):
        return self.subsegment.SizeIndex

    def getBlockCount(self):
        return self.subsegment.BlockCount

    def getFreeBlockCount(self):
        return self.subsegment.AggregateExchg.Depth

    def getUsedBlockCount(self):
        return self.getBlockCount() - self.getFreeBlockCount()

class Win32LFH(object):
    def __init__(self, trace, heap, address):
        self.trace = trace
        self.heap = heap
        self.address = address
        self.lfh = trace.getStruct('ntdll.LFH_HEAP', address)

    def getSubsegments(self):
        for num, localdata in self.lfh.LocalData:
            for bucket, seginfo in localdata.SegmentInfo:
                if seginfo.ActiveSubsegment == 0:
                    continue
                yield Win32Subsegment(self.trace, self.heap, seginfo.ActiveSubsegment)



import vdb
import cobra
import getpass


defconfig = {
    'viv':{

        'SymbolCacheSave':True,

        'parsers':{
            'pe':{
                'baseaddr': 0x200000,
                'loadresources':False,
                'carvepes':True,
                'nx':False,
            },
            'elf':{
                'baseaddr': 0x200000,
            },
            'blob':{
                'arch':'',
                'bigend':False,
                'baseaddr':0x20200000,
            },
            'macho':{
                'baseaddr':0x70700000,
                'fatarch':'',
            },
            'ihex':{
                'arch':'',
                'offset':0,
            },
            'srec':{
                'arch':'',
                'offset':0,
            },
        },
        'analysis':{
            'pointertables':{
                'table_min_len':4,
            },
            'symswitchcase':{
                'max_instr_count': 10,
                'max_cases': 500,
                'case_failure': 5000,
                'min_func_instr_size': 10,
                'timeout_secs': 45,
            }
        },
        'remote':{
            'wait_for_plat_arch': 10,
            'timeout_wait': 10,
            'timeout_aban': 120,
        },
        'server':{
            'queue_chunksize': 70000,
        },
    },
    'cli':vdb.defconfig.get('cli'), # FIXME make our own...
    'vdb':vdb.defconfig.get('vdb'),
    'cobra':cobra.defconfig.get('cobra'),
    'user':{
        'name': getpass.getuser(),
    }
}

defconfig.get('cli').update(vdb.defconfig.get('cli'))

# Config elements docs
docconfig = {

    'viv':{

        'SymbolCacheSave':'Save vivisect names to the vdb configured symbol cache?',

        'parsers':{
            'pe':{
                'baseaddr': 'Address used to relocate PE files if base-address is 0 and PE is relocatable',
                'loadresources':'Should we load resource segments?',
                'carvepes':'Should we carve pes?',
                'nx':'Should we truly treat sections that dont execute as non executable?'
            },
            'elf':{
                'baseaddr': 'Address used to relocate ELF files if base-address is 0 and ELF is relocatable',
            },
            'blob':{
                'arch':'What architecture is the blob?',
                'baseaddr':'Base virtual address for loading the blob.',
            },
            'macho':{
                'baseaddr':'Base virtual address for loading the macho',
                'fatarch':'Which architecture binary to extract from a FAT macho',
            },
            'ihex':{
                'arch':'What architecture is the ihex dump?',
            },
            'srec':{
                'arch':'What architecture is the srec dump?',
                'offset':'Skip over initial bytes in the file',
            },
        },

        'analysis':{
            'pointertables':{
                'table_min_len':'How many pointers must be in a row to make a table?',
            },
        },
        'remote':{
            'wait_for_plat_arch':'How many secs to wait for the remote server/workspace to provide a Platform or Architecture before moving on.',
            'timeout_wait': "Timeout waiting for getNextEvent() to have more Viv events to send.",
            'timeout_aban': "Server channel timeout.  At this point, clean up and delete the channel.  The connection is dead.",
        },

        'server':{
            'queue_chunksize':"VivServer Queue Chunk Size, the largest chunk of events the server will send at a time.  This affects queue time and overall efficiency of serving large workspaces",
        },
    },

    'vdb':vdb.docconfig.get('vdb'),
    'user':{
        'name': 'Username.  When not set, defaults to current system user.',
    }
}



import envi
import envi.memory as e_mem


class WorkspaceMemoryObject(e_mem.MemoryObject):

    def __init__(self, vw, maps, nosegfault=False):
        self.vw = vw
        self.nosegfault = nosegfault
        e_mem.MemoryObject.__init__(self, maps)

    # FIXME make this copy on write from the workspace

    def readMemory(self, va, size):
        if self.checkMemory(va):
            return e_mem.MemoryObject.readMemory(self, va, size)
        if self.vw.getSegment(va) is not None:
            return self.vw.readMemory(va, size)
        # We don't have it
        if self.nosegfault:
            return "A" * size
        raise envi.SegmentationViolation(va)



from envi.const import *
from envi import IF_NOFALL, IF_PRIV, IF_CALL, IF_BRANCH, IF_RET, IF_COND

IF_BYTE = 1<<8

# No operand instructions
nocode = [
    '.word' # Something is wrong, so return the dirty word
]

# Single operand intructions
scode = [
    ('rrc', 0),          # RRC Rotate right through carry
    ('swpb', 0),         # SWPB Swap bytes
    ('rra', 0),          # RRA Rotate right arithmetic
    ('sxt', 0),          # SXT Sign extend byte to word
    ('push', 0),         # PUSH Push value onto stack
    ('call', IF_CALL),   # CALL Subroutine call; push PC and move source to PC
    ('reti', IF_NOFALL), # RETI Return from interrupt; pop SR then pop PC
]

# Jump conditions
jcode = [
    ('jnz', IF_BRANCH | IF_COND),   # JNE/JNZ Jump if not equal/zero
    ('jz', IF_BRANCH | IF_COND),    # JEQ/JZ Jump if equal/zero
    ('jnc', IF_BRANCH | IF_COND),   # JNC/JLO Jump if no carry/lower
    ('jc', IF_BRANCH | IF_COND),    # JC/JHS Jump if carry/higher or same
    ('jn', IF_BRANCH | IF_COND),    # JN Jump if negative
    ('jge', IF_BRANCH | IF_COND),   # JGE Jump if greater or equal
    ('jl', IF_BRANCH | IF_COND),    # JL Jump if less
    ('jmp', IF_BRANCH | IF_NOFALL), # JMP Jump (unconditionally)
]

# Double operand instrucitons
dcode = [
    'mov',  # MOV Move source to destination
    'add',  # ADD Add source to destination
    'addc', # ADDC Add source and carry to destination
    'subc', # SUBC Subtract source from destination (with carry)
    'sub',  # SUB Subtract source from destination
    'cmp',  # CMP Compare (pretend to subtract) source from destination
    'dadd', # Decimal add source to destination (with carry)
    'bit',  # BIT Test bits of source AND destination
    'bic',  # BIC Bit clear (dest &= ~src)
    'bis',  # BIS Bit set (logical OR)
    'xor',  # XOR Exclusive or source with destination
    'and'   # AND Logical AND source with destination (dest &= src)
]

# Double special operand instructions
dspcode = [
    ('nop', 0),                  # No Operation - MOV
    ('pop', 0),                  # POP stackpointer - MOV
    ('br', IF_BRANCH|IF_NOFALL), # Branch - MOV
    ('ret', IF_NOFALL),          # Return - MOV
    ('clr', 0),                  # Clear destination - MOV
    ('rla', 0),                  # Shift and rotate left - ADD
    ('inc', 0),                  # Increment by one - ADD
    ('incd', 0),                 # Increment by two - ADD
    ('rlc', 0),                  # Shift and rotate left - ADDC
    ('adc', 0),                  # Adding only the carry bit - ADDC
    ('sbc', 0),                  # Subtracting only the carry bit - SUBC
    ('dec', 0),                  # Decrement by one - SUB
    ('decd', 0),                 # Decrement by two - SUB
    ('tst', 0),                  # Test - CMP
    ('dadc', 0),                 # Decimal adding only the carry bit - DADD
    ('clrc', 0),                 # Status register operation - BIC
    ('setc', 0),                 # Status register operation - BIS
    ('clrz', 0),                 # Status register operation - BIC
    ('setz', 0),                 # Status register operation - BIS
    ('clrn', 0),                 # Status register operation - BIC
    ('setn', 0),                 # Status register operation - BIS
    ('dint', 0),                 # Status register operation - BIC
    ('eint', 0),                 # Status register operation - BIC
    ('inv', 0),                  # Invert value - XOR
]

# Operand Type
SINGLE_OPCODE_TYPE = 0
JUMP_OPCODE_TYPE   = 1
DOUBLE_OPCODE_TYPE = 2
SP_OPCODE_TYPE     = 3

# Register Modes
REG_DIRECT      = 0x0
REG_INDEX       = 0x1
REG_INDIRECT    = 0x2
REG_IND_AUTOINC = 0x3
JUMP_MODE       = 0x4

# Masks
TEST_MASKS       = 0xF000 # Test operands
SINGLE_MASKS     = 0xE000 # ID single operands
JUMP_MASKS       = 0xC000 # ID jumps
JUMP_OFFSET      = 0x3FF  # Jump offset
SOURCE_REG       = 0xF    # Single op reg
DSOURCE_REG      = 0xF00  # Double source reg
DEST_REG         = 0xF    # Double dest reg
BYTE_WORD        = 0x40   # Byte or word
SOURCE_ADDR_MODE = 0x30   # Addressing mode source
DEST_ADDR_MODE   = 0x80   # Addressing mode destination
RETI_MASK        = 0x1300 # Return 'reti'
REG_BYTE         = 0x00FF # Clear the most significant byte
REG_FLAGS        = 0x01FF # Clear most significant 7 bits to get Status Register flags

# Compare to get the proper Opcode
SINGLE_OPCODE   = 0xF80   # Single opcode - rotate right 7
DOUBLE_OPCODE   = 0xF000  # Double opcode - rotate right 12
JUMP_OPCODE     = 0x1c00  # Jump condition - rotate right 10

# Sizes
BYTE = 1 # bytes
WORD = 2 # bytes



import logging
import unittest

import Elf
import envi
import vivisect.cli as viv_cli
import vivisect.tests.helpers as helpers

logger = logging.getLogger(__name__)

path = ('raw', 'arm', 'dummy.blob')


class BlobTests(unittest.TestCase):

    def test_blob(self):
        fn = helpers.getTestPath(*path)
        vw = viv_cli.VivCli()
        vw.config.viv.parsers.blob.arch = 'arm'
        vw.config.viv.parsers.blob.baseaddr = 0x200000
        vw.loadFromFile(fn)

        vw.makeFunction(0x200001)
        vw.makeFunction(0x200008)

        self.assertEqual(vw.getFunction(0x200007), 0x200000)
        self.assertEqual(vw.getFunction(0x200018), 0x200008)




from envi.const import *

REG_PC = 8
REG_SP = 7
REG_FLAGS = 9


# opcode flags
IF_B = 0x100
IF_W = 0x200
IF_L = 0x400

OSZ_FLAGS = (
        None,
        IF_B,
        IF_W,
        None,
        IF_L,
        )

# operand flags
OF_PREDEC = 1
OF_POSTINC = 2



import sys
import copy
import struct

import envi
import envi.bits as e_bits
import envi.memory as e_mem

from envi.archs.msp430.regs import *

########################################################
# Process special opcodes that come from double opcodes
#
# These could use the dpbase parse from arm.py
#

def dpbase(workData):
    """
    Parse and return dcode_val, dsreg, ds_addr_mode, 
    destreg, dest_addr_mode, op_bw, dsopsize, destopsize
    for a standard data processing instruction.
    """
    dcode_val = ((workData & DOUBLE_OPCODE) >> 12) - 4
    dsreg = (workData & DSOURCE_REG) >> 8
    ds_addr_mode = (workData & SOURCE_ADDR_MODE) >> 4
    destreg = workData & DEST_REG
    dest_addr_mode = (workData & DEST_ADDR_MODE) >> 7
    op_bw = (workData & BYTE_WORD) << 2     # IF_BYTE
    if op_bw:
        dsopsize = 1
        destopsize = 1
    else:
        dsopsize = 2
        destopsize = 2

    # FIXME: wtf is this logic?  where should op_bw play here?
    #if (ds_addr_mode == REG_INDEX) or (ds_addr_mode == REG_IND_AUTOINC):
    #    dsopsize = 1
    #if (dest_addr_mode == REG_INDEX):
    #    destopsize = 1
    return dcode_val, dsreg, ds_addr_mode, destreg, dest_addr_mode, op_bw, dsopsize, destopsize

def decode0(workData, opData, va):
    # Parse out because of special operations
    # MOV
    dcode_val, dsreg, ds_addr_mode, destreg, dest_addr_mode, op_bw, dsopsize, destopsize = dpbase(workData)

    # NOP
    if ((dsreg > 3) and (dsreg == destreg)) and not ds_addr_mode and not dest_addr_mode:
        mnem, flags = dspcode[0]
        return Msp430Opcode( va, SP_OPCODE_TYPE, mnem, [], 0, opData.lenData())
    # RET
    # moved before POP to process properly
    if ((dsreg == 1) and (ds_addr_mode == REG_IND_AUTOINC) and (destreg == 0)):
        mnem, flags = dspcode[3]
        return Msp430Opcode( va, SP_OPCODE_TYPE, mnem, [ ], IF_NOFALL, opData.lenData())
    # POP
    if ((dsreg == 1) and (ds_addr_mode == REG_IND_AUTOINC)):
        mnem, flags = dspcode[1]
        return Msp430Opcode( va, SINGLE_OPCODE_TYPE, mnem, [ Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ], op_bw, opData.lenData())
    # BRANCH
    if (destreg == 0) and not dest_addr_mode:
        mnem, flags = dspcode[2]
        op = Msp430Opcode( va, SINGLE_OPCODE_TYPE, mnem, [ Msp430Operands[ds_addr_mode](dsreg, opData, dsopsize, va) ], IF_BRANCH|IF_NOFALL, opData.lenData())
        return op
    '''
    # CLR
    if ((dsreg == 3) and (ds_addr_mode == REG_DIRECT)):
    mnem, flags = dspcode[4]
        return Msp430Opcode( va, SINGLE_OPCODE_TYPE, mnem, [ Msp430Operands[dest_addr_mode](destreg, opData, destopsize) ], op_bw)
    '''
    # Nothing Special
    return Msp430Opcode(
            va,
            DOUBLE_OPCODE_TYPE,
            dcode[dcode_val],
            [ Msp430Operands[ds_addr_mode](dsreg, opData, dsopsize, va), Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ],
            op_bw, 
            opData.lenData()
            )

def decode1(workData, opData, va):
    # Parse out because of special operations
    # ADD
    dcode_val, dsreg, ds_addr_mode, destreg, dest_addr_mode, op_bw, dsopsize, destopsize = dpbase(workData)
    
    # RLA
    if (dsreg == destreg) and not ds_addr_mode and not dest_addr_mode:
        mnem, flags = dspcode[5]
        return Msp430Opcode( va, SINGLE_OPCODE_TYPE, mnem, [ Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ], op_bw, opData.lenData())
    # INC
    if ((dsreg == 3) and (ds_addr_mode == REG_INDEX)):
        mnem, flags = dspcode[6]
        return Msp430Opcode( va, SINGLE_OPCODE_TYPE, mnem, [ Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ], op_bw, opData.lenData())
    # INCD
    if ((dsreg == 3) and (ds_addr_mode == REG_INDIRECT)):
        mnem, flags = dspcode[7]
        return Msp430Opcode( va, SINGLE_OPCODE_TYPE, mnem, [ Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ], op_bw, opData.lenData())
    # Nothing Special
    return Msp430Opcode(
            va,
            DOUBLE_OPCODE_TYPE,
            dcode[dcode_val],
            [ Msp430Operands[ds_addr_mode](dsreg, opData, dsopsize, va), Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ],
            op_bw,
            opData.lenData())

def decode2(workData, opData, va):
    # Parse out because of special operations
    # ADDC
    dcode_val, dsreg, ds_addr_mode, destreg, dest_addr_mode, op_bw, dsopsize, destopsize = dpbase(workData)
    
    # RLC
    if (dsreg == destreg) and not ds_addr_mode and not dest_addr_mode:
        mnem, flags = dspcode[8]
        return Msp430Opcode( va, SINGLE_OPCODE_TYPE, mnem, [ Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ], op_bw, opData.lenData())
    # ADC
    if ((dsreg == 3) and (ds_addr_mode == REG_DIRECT)):
        mnem, flags = dspcode[9]
        return Msp430Opcode( va, SINGLE_OPCODE_TYPE, mnem, [ Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ], op_bw, opData.lenData())
    # Nothing Special
    return Msp430Opcode(
            va,
            DOUBLE_OPCODE_TYPE,
            dcode[dcode_val],
            [ Msp430Operands[ds_addr_mode](dsreg, opData, dsopsize, va), Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ],
            op_bw,
            opData.lenData())

def decode3(workData, opData, va):
    # Parse out because of special operations
    # SUBC
    dcode_val, dsreg, ds_addr_mode, destreg, dest_addr_mode, op_bw, dsopsize, destopsize = dpbase(workData)
    
    # SBC
    if ((dsreg == 3) and (ds_addr_mode == REG_DIRECT)):
        mnem, flags = dspcode[10]
        return Msp430Opcode( va, SINGLE_OPCODE_TYPE, mnem, [ Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ], op_bw, opData.lenData())
    # Nothing Special
    return Msp430Opcode(
            va,
            DOUBLE_OPCODE_TYPE,
            dcode[dcode_val],
            [ Msp430Operands[ds_addr_mode](dsreg, opData, dsopsize, va), Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ],
            op_bw,
            opData.lenData())

def decode4(workData, opData, va):
    # Parse out because of special operations
    # SUB
    dcode_val, dsreg, ds_addr_mode, destreg, dest_addr_mode, op_bw, dsopsize, destopsize = dpbase(workData)
    
    # DEC
    if ((dsreg == 3) and (ds_addr_mode == REG_INDEX)):
        mnem, flags = dspcode[11]
        return Msp430Opcode( va, SINGLE_OPCODE_TYPE, mnem, [ Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ], op_bw, opData.lenData())
    # DECD
    if ((dsreg == 3) and (ds_addr_mode == REG_INDIRECT)):
        mnem, flags = dspcode[12]
        return Msp430Opcode( va, SINGLE_OPCODE_TYPE, mnem, [ Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ], op_bw, opData.lenData())
    # Nothing Special
    return Msp430Opcode(
            va,
            DOUBLE_OPCODE_TYPE,
            dcode[dcode_val],
            [ Msp430Operands[ds_addr_mode](dsreg, opData, dsopsize, va), Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ],
            op_bw,
            opData.lenData())

def decode5(workData, opData, va):
    # Parse out because of special operations
    # CMP
    dcode_val, dsreg, ds_addr_mode, destreg, dest_addr_mode, op_bw, dsopsize, destopsize = dpbase(workData)
    
    # TST
    if ((dsreg == 3) and (ds_addr_mode == REG_DIRECT)):
        mnem, flags = dspcode[13]
        return Msp430Opcode( va, SINGLE_OPCODE_TYPE, mnem, [ Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ], op_bw, opData.lenData())
    # Nothing Special
    return Msp430Opcode(
            va,
            DOUBLE_OPCODE_TYPE,
            dcode[dcode_val],
            [ Msp430Operands[ds_addr_mode](dsreg, opData, dsopsize, va), Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ],
            op_bw,
            opData.lenData())

def decode6(workData, opData, va):
    # Parse out because of special operations
    # DADD
    dcode_val, dsreg, ds_addr_mode, destreg, dest_addr_mode, op_bw, dsopsize, destopsize = dpbase(workData)
    
    # DADC
    if ((dsreg == 3) and (ds_addr_mode == REG_DIRECT)):
        mnem, flags = dspcode[14]
        return Msp430Opcode( va, SINGLE_OPCODE_TYPE, mnem, [ Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ], op_bw, opData.lenData())
    # Nothing Special
    return Msp430Opcode(
            va,
            DOUBLE_OPCODE_TYPE,
            dcode[dcode_val],
            [ Msp430Operands[ds_addr_mode](dsreg, opData, dsopsize, va), Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ],
            op_bw,
            opData.lenData())

def decode7(workData, opData, va):
    # Parse out because of special operations
    # Nothing Special, move along now
    dcode_val, dsreg, ds_addr_mode, destreg, dest_addr_mode, op_bw, dsopsize, destopsize = dpbase(workData)
    return Msp430Opcode(
            va,
            DOUBLE_OPCODE_TYPE,
            dcode[dcode_val],
            [ Msp430Operands[ds_addr_mode](dsreg, opData, dsopsize, va), Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ],
            op_bw,
            opData.lenData())

def decode8(workData, opData, va):
    # Parse out because of special operations
    # BIC
    dcode_val, dsreg, ds_addr_mode, destreg, dest_addr_mode, op_bw, dsopsize, destopsize = dpbase(workData)
    
    # CLRC
    if ((destreg == 2) and (dsreg == 3) and (ds_addr_mode == REG_INDEX)):
        mnem, flags = dspcode[15]
        return Msp430Opcode( va, SP_OPCODE_TYPE, mnem, [ ], 0, opData.lenData())
    # CLRZ
    if ((destreg == 2) and (dsreg == 3) and (ds_addr_mode == REG_INDIRECT)):
        mnem, flags = dspcode[17]
        return Msp430Opcode( va, SP_OPCODE_TYPE, mnem, [ ], 0, opData.lenData())
    # CLRN
    if ((destreg == 2) and (dsreg == 2) and (ds_addr_mode == REG_INDIRECT)):
        mnem, flags = dspcode[19]
        return Msp430Opcode( va, SP_OPCODE_TYPE, mnem, [ ], 0, opData.lenData())
    # DINT
    if ((destreg == 2) and (dsreg == 2) and (ds_addr_mode == REG_IND_AUTOINC)):
        mnem, flags = dspcode[21]
        return Msp430Opcode( va, SP_OPCODE_TYPE, mnem, [ ], 0, opData.lenData())
    # Nothing Special
    return Msp430Opcode(
            va,
            DOUBLE_OPCODE_TYPE,
            dcode[dcode_val],
            [ Msp430Operands[ds_addr_mode](dsreg, opData, dsopsize, va), Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ],
            op_bw,
            opData.lenData()
            )

def decode9(workData, opData, va):
    # Parse out because of special operations
    # BIS
    dcode_val, dsreg, ds_addr_mode, destreg, dest_addr_mode, op_bw, dsopsize, destopsize = dpbase(workData)
    
    # SETC
    if ((destreg == 2) and (dsreg == 3) and (ds_addr_mode == REG_INDEX)):
        mnem, flags = dspcode[16]
        return Msp430Opcode( va, SP_OPCODE_TYPE, mnem, [ ], 0, opData.lenData())
    # SETZ
    if ((destreg == 2) and (dsreg == 3) and (ds_addr_mode == REG_INDIRECT)):
        mnem, flags = dspcode[18]
        return Msp430Opcode( va, SP_OPCODE_TYPE, mnem, [ ], 0, opData.lenData())
    # SETN
    if ((destreg == 2) and (dsreg == 2) and (ds_addr_mode == REG_INDIRECT)):
        mnem, flags = dspcode[20]
        return Msp430Opcode( va, SP_OPCODE_TYPE, mnem, [ ], 0, opData.lenData())
    # DINT
    if ((destreg == 2) and (dsreg == 2) and (ds_addr_mode == REG_IND_AUTOINC)):
        mnem, flags = dspcode[22]
        return Msp430Opcode( va, SP_OPCODE_TYPE, mnem, [ ], 0, opData.lenData())
    # Nothing Special
    return Msp430Opcode(
            va, 
            DOUBLE_OPCODE_TYPE,
            dcode[dcode_val],
            [ Msp430Operands[ds_addr_mode](dsreg, opData, dsopsize, va), Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ],
            op_bw,
            opData.lenData()
            )

def decode10(workData, opData, va):
    # Parse out because of special operations
    # XOR
    dcode_val, dsreg, ds_addr_mode, destreg, dest_addr_mode, op_bw, dsopsize, destopsize = dpbase(workData)
    
    # INV
    if ((dsreg == 3) and (ds_addr_mode == REG_IND_AUTOINC)):
        mnem, flags = dspcode[23]
        return Msp430Opcode( va, SINGLE_OPCODE_TYPE, mnem, [ Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ], op_bw, opData.lenData())
    # Nothing Special
    return Msp430Opcode(
            va, 
            DOUBLE_OPCODE_TYPE,
            dcode[dcode_val],
            [ Msp430Operands[ds_addr_mode](dsreg, opData, dsopsize, va), Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ],
            op_bw,
            opData.lenData()
            )

def decode11(workData, opData, va):
    # Parse out because of special operations
    # Nothing Special, move along now
    dcode_val, dsreg, ds_addr_mode, destreg, dest_addr_mode, op_bw, dsopsize, destopsize = dpbase(workData)
    
    return Msp430Opcode(
            va, 
            DOUBLE_OPCODE_TYPE,
            dcode[dcode_val],
            [ Msp430Operands[ds_addr_mode](dsreg, opData, dsopsize, va), Msp430Operands[dest_addr_mode](destreg, opData, destopsize, va) ],
            op_bw,
            opData.lenData()
            )

######################################################
# Table of the parser functions for special opcodes

decode_function = (
    decode0,
    decode1,
    decode2,
    decode3,
    decode4,
    decode5,
    decode6,
    decode7,
    decode8,
    decode9,
    decode10,
    decode11
)

class Msp430Opcode(envi.Opcode):

    def __init__(self, va, opcode, mnem, opers, iflags=0, size=0):
        self.va = va
        self.opcode = opcode
        self.mnem = mnem
        self.opers = opers
        self.iflags = iflags | envi.ARCH_MSP430
        self.size = size

    def __len__(self):
        return self.size

    def isByte(self):
        return self.iflags

    def render(self, mcanv):
        """
        Render this opcode to the specified memory canvas
        """
        mnem = self.mnem
        if self.iflags & IF_BYTE:
            mnem += '.b'

        mcanv.addNameText(mnem, typename="mnemonic")
        mcanv.addText(" ")

        # Allow each of our operands to render
        imax = len(self.opers)
        lasti = imax - 1
        for i in range(imax):
            oper = self.opers[i]
            oper.render(mcanv, self, i)
            if i != lasti:
                mcanv.addText(", ")

    def __repr__(self):
        if self.iflags & IF_BYTE:
            # Let everybody know that we only need a byte instead of a word
            mnem = self.mnem + '.b'
        else:
            mnem = self.mnem

        x = [mnem]
        # Test for Special Opcode Type and 'reti'
        if  self.opcode == SP_OPCODE_TYPE: 
            return " ".join(x)
        if self.opcode == DOUBLE_OPCODE_TYPE:
            x.append(repr(self.opers[0]) + ",")
            x.append(repr(self.opers[1]))
        else:
            # Single opcode or jumps
            x.append(repr(self.opers[0]))
        return " ".join(x)

    def getBranches(self, emu=None):
        ret = []

        # To start with we have no flags ( except our arch )
        flags = self.iflags & envi.ARCH_MASK
        addb = False

        # If we are a conditional branch, even our fallthrough
        # case is conditional...
        if (self.iflags & IF_BRANCH):
            addb = True
            if not (self.iflags & IF_NOFALL):
                flags |= envi.BR_COND

        # If we can fall through, reflect that...
        if not self.iflags & envi.IF_NOFALL:
            ret.append((self.va + self.size, flags|envi.BR_FALL))

        # if we have no operands, it has no further branches...
        if len(self.opers) == 0:
            return ret

        # Check for a call...
        if self.iflags & IF_CALL:
            flags |= envi.BR_PROC
            addb = True

        if addb:
            oper0 = self.opers[0]
            if oper0.isDeref():
                flags |= envi.BR_DEREF
                tova = oper0.getOperAddr(self, emu=emu)
            else:
                tova = oper0.getOperValue(self, emu=emu)

            ret.append((tova, flags))

        return ret

#FIXME: This should just be wrapped into envi
def addrToName(mcanv, va):
    sym = mcanv.syms.getSymByAddr(va)
    if sym is not None:
        return repr(sym)
    return "0x%.4x" % va

def renderPossibleLocation(mcanv, op, idx, value):
    hint = mcanv.syms.getSymHint(op.va, idx)
    if hint is not None:
        if mcanv.mem.isValidPointer(value):
            mcanv.addVaText(hint, value)
        else:
            mcanv.addNameText(hint)
    elif mcanv.mem.isValidPointer(value):
        name = addrToName(mcanv, value)
        mcanv.addVaText(name, value)
    else:
        value &= 0xffff
        mcanv.addNameText('0x%x' % value)

class Msp430Operand(envi.Operand):

    def __init__(self, val, inData, tsize=2, va=0):
        self.val = val
        self.tsize = tsize
        self.va = va

    def render(self, mcanv, op, idx):
        mcanv.addText(" " + self.__repr__())

    def renderReg(self, mcanv, op, idx, reg):
        hint = mcanv.syms.getSymHint(op.va, idx)
        if hint is not None:
            mcanv.addNameText(hint, typename="registers")
        else:
            name = self._dis_regctx.getRegisterName(reg)
            rname = self._dis_regctx.getRegisterName(reg&RMETA_NMASK)
            mcanv.addNameText(name, name=rname, typename="registers")

class Msp430RegDirectOper(Msp430Operand):
    def __repr__(self):
        # Register direct
        if self.val == REG_CG:
            # Special Case
            return "#0"
        else:
            # return Rn
            regname = registers[self.val]
            return "%s" % regname

    def render(self, mcanv, op, idx):
        if self.val == REG_CG:
            mcanv.addText("#0x0")
            return

        self.renderReg(mcanv, op, idx, self.val)


    def getOperValue(self, op, emu=None):
        if self.val == REG_PC:
            return op.va + op.size
        if self.val == REG_CG:
            return 0

        if emu==None: 
            return

        return emu.getRegister(self.val)

    def setOperValue(self, op, emu, val):
        if self.val == REG_CG:
            return 0

        return emu.setRegister(self.val, val)

class Msp430RegIndexOper(Msp430Operand):
    def __init__(self, val, inData, tsize=0, va=0):
        Msp430Operand.__init__(self, val, inData, tsize, va)
        if val != REG_CG:
            new_val = inData.nextData()

            # this is signed
            if new_val > 32768:
                new_val = ((new_val & 32767) - 32768)

            self.new_val = new_val

    def __repr__(self):
        # Register indexed
        if self.val == REG_SR:
            # Special Case
            return "&0x%0.4x" % self.new_val
        if self.val == REG_CG:
            # Special Case
            return "#1"
        new_val = self.new_val
        # If r0 then return hex value
        if not self.val:
            return "0x%0.4x" % new_val

        regname = registers[self.val]
        return "0x%x(%s)" % (new_val,regname)

    def render(self, mcanv, op, idx):
        if self.val == REG_SR:
            mcanv.addText('&')
            renderPossibleLocation(mcanv, op, idx, self.new_val)

        elif self.val == REG_CG:
            mcanv.addText('#0x1')

        elif self.val == REG_PC:
            mcanv.addText('#')
            renderPossibleLocation(mcanv, op, idx, self.new_val)

        else:
            if self.new_val < 0:
                preface = '-0x%x(' % abs(self.new_val)
            else:
                preface = '0x%x(' % self.new_val
            mcanv.addText(preface)
            self.renderReg(mcanv, op, idx, self.val)
            mcanv.addText(')')

    def setOperValue(self, op, emu, val):
        if self.val == REG_CG:
            return 1

        addr = self.getOperAddr(op, emu)
        emu.writeMemValue(addr, val, self.tsize)

    def getOperValue(self, op, emu=None):
        if self.val == REG_CG:
            return 1

        if emu==None:
            return None

        addr = self.getOperAddr(op, emu)
        val = emu.readMemValue(addr, self.tsize)
        return val


    def getOperAddr(self, op, emu=None):
        if self.val == REG_SR:
            return self.new_val
        if self.val == REG_PC:
            return op.va + op.size + self.new_val

        if emu==None:
            return None

        addr = emu.getRegister(self.val)
        return addr + self.new_val

    def isDeref(self):
        if self.val == REG_CG:
            return False
        return True

class Msp430RegIndirOper(Msp430Operand):
    def __repr__(self):
        # Register indirect
        if self.val == REG_SR:
            # Special Case
            return "#0x4"
        if self.val == REG_CG:
            # Special Case
            return "#0x2"
        regname = registers[self.val]
        return "@%s" % regname

    def render(self, mcanv, op, idx):
        # Register indirect
        if self.val == REG_SR:
            # Special Case
            mcanv.addText("#")
            mcanv.addNameText("0x4")
            return

        if self.val == REG_CG:
            # Special Case
            mcanv.addText("#")
            mcanv.addNameText("0x2")
            return

        mcanv.addText("@")
        self.renderReg(mcanv, op, idx, self.val)

    def setOperValue(self, op, emu, val):
        if self.val in (REG_SR, REG_CG):
            return

        addr = self.getOperAddr(op, emu)
        emu.writeMemValue(addr, val, self.tsize)

    def getOperValue(self, op, emu=None):
        if self.val in (REG_SR, REG_CG):
            return (None, None, 4, 2)[self.val]

        if emu is None:
            return None

        addr = self.getOperAddr(op, emu)
        val = emu.readMemValue(addr, self.tsize)
        return val

    def getOperAddr(self, op, emu=None):
        if self.val == REG_PC:
            return op.va + op.size

        if emu is None:
            return None

        return emu.getRegister(self.val)

    def isDeref(self):
        if self.val in (REG_SR, REG_CG):
            return False
        return True

class Msp430RegIndirAutoincOper(Msp430Operand):
    def __init__(self, val, inData, tsize, va=0):
        Msp430Operand.__init__(self, val, inData, tsize, va)
        if val == REG_PC:
            new_val = inData.nextData()
            if new_val > 32768:
                new_val = ((new_val & 32767) - 32768)

            self.new_val = new_val

    def __repr__(self):
        # Register indirect autoincrement
        if self.val == REG_PC:
            # Return @PC+x where x is the next word in the instruction stream
            return "#0x%x" % (self.new_val)

        if self.val == REG_SR:
            # Special Case
            return "#0x8"
        if self.val == REG_CG:
            # Special Case
            return "#-0x1"

        regname = registers[self.val]
        return "@%s+" % regname

    def render(self, mcanv, op, idx):
        # Register indirect autoincrement
        if self.val == REG_PC:
            # Return @PC+x where x is the next word in the instruction stream
            mcanv.addText("#")
            renderPossibleLocation(mcanv, op, idx, self.new_val)
            return

        if self.val == REG_SR:
            mcanv.addText("#")
            mcanv.addNameText("0x8")
            return

        if self.val == REG_CG:
            mcanv.addText("#")
            mcanv.addNameText("-0x1")
            return

        self.renderReg(mcanv, op, idx, self.val)

    def setOperValue(self, op, emu, val):
        if self.val in (REG_PC, REG_SR, REG_CG):
            return 

        addr = self.getOperAddr(op, emu)
        emu.readMemValue(addr, va, self.tsize)

    def getOperValue(self, op, emu=None):
        if self.val == REG_PC:
            return self.new_val
        elif self.val == REG_SR:
            return 8
        elif self.val == REG_CG:
            return -1

        if emu is None:
            return None
        addr = self.getOperAddr(op, emu)
        val = emu.readMemValue(addr, self.tsize)
        return val

    def getOperAddr(self, op, emu=None):
        if emu is None:
            return None

        addr = emu.getRegister(self.val)
        emu.setRegister(self.val, addr + self.tsize)
        return addr

    def isDeref(self):
        if self.val in (REG_PC, REG_SR, REG_CG):
            return False
        return True

class Msp430JmpOper(Msp430Operand):
    def __init__(self, val, inData, tsize, va=0):
        if (val > 0xff):
            jmp_val = va + (2 * ((val & 511) - 512)) + 2
        else:
            jmp_val = va + (2 * val) + 2

        Msp430Operand.__init__(self, jmp_val, inData, tsize, va)

    def __repr__(self):
            # Process and sign jump offsets
            # PC = PC + 2 * offset
            if self.va:
                return "#0x%x" % self.val
            return "$0x%x" % self.val

    def render(self, mcanv, op, idx):
        if self.va:
            mcanv.addText("#")
        else:
            mcanv.addText("$")

        renderPossibleLocation(mcanv, op, idx, self.val)

    def setOperValue(self, op, emu, val):
        return

    def getOperValue(self, op, emu=None):
        return self.val


class Msp430Disasm:
    def __init__(self):
        self._dis_regctx = Msp430RegisterContext()
        self._dis_oparch = envi.ARCH_MSP430

    # FIXME: Msp430Data is kinda kludgy and should be wrapped into the disasm logic
    class Msp430Data:
        """
            Msp430Data processes the incoming data.
            Optimally this will be 6 words coming in
            but, it could be more.
        """
        def __init__ (self, inData, inPos):
            self.data = inData
            self.offset = self.start = inPos
            self.next = inPos + 2
    
        def getData(self):
            return struct.unpack("<H", self.data[self.offset:self.next])[0]
    
        def nextData(self):
            self.incData()
            return struct.unpack("<H", self.data[self.offset:self.next])[0]

        def incData(self):
            self.offset += 2
            self.next += 2
            return

        def lenData(self):
            return (self.next - self.start)

    def disasm(self, bytes, offset=0, va=0):
        opData = self.Msp430Data(bytes, offset)
        while not opData.lenData() == 0:
            workData = opData.getData()
            if (workData >> 13):
                if ((workData >> 13) > 1):
                    # Parse out because of special operations
                    # This doesn't see to allow returning the function
                    # Have to set a variable
                    result =  decode_function[(((workData & DOUBLE_OPCODE) >> 12) - 4)](workData, opData, va)
                    # FILTHY HACK
                    for oper in result.opers:
                        oper._dis_regctx = self._dis_regctx
                    return result
                else:
                    # It is a Jump Opcode
                    mnem, flags = jcode[(workData & JUMP_OPCODE) >> 10]
                    op = Msp430Opcode(
                            va,
                            JUMP_OPCODE_TYPE,
                            mnem, 
                            [ Msp430JmpOper((workData & JUMP_OFFSET), opData, 0, va) ],
                            flags,
                            opData.lenData()
                            )
                    # FILTHY HACK
                    for oper in op.opers:
                        oper._dis_regctx = self._dis_regctx
                    return op
            else:
                # Test to see of primary input is not opcode
                if not (workData >> 12):
                    # Okay, something went wrong, so return the dirty word
                    raise Exception("Decode Error 0")

                ds_addr_mode = (workData & SOURCE_ADDR_MODE) >> 4
                dsreg = workData & DEST_REG
                op_bw = (workData & BYTE_WORD) >> 6
                dsopsize = 0
                # Test op_dw here to ensure it is used correctly
                if (((workData & SINGLE_OPCODE) >> 7) in [1,3,5,6]) and op_bw:
                    # Okay, something went wrong, so return the dirty word
                    raise Exception("Decode Error 1")
                if (ds_addr_mode == REG_INDEX) or (ds_addr_mode == REG_IND_AUTOINC):
                    dsopsize = 1
                # It is a Single Opcode
                # Process reti so don't have to do a 
                # string compare on mnem during repr
                if (((workData & SINGLE_OPCODE) >> 7) == 6):
                    if (workData & RETI_MASK):
                        mnem, flags = scode[6]
                        return Msp430Opcode( va, SP_OPCODE_TYPE, mnem, [], flags, opData.lenData())
                    else:
                        # Okay, something went wrong, so return the dirty word
                        raise Exception('Decode Error 2')
                else:
                    mnem, flags = scode[(workData & SINGLE_OPCODE) >> 7]
                    flags |= (op_bw << 8)
                    op = Msp430Opcode(
                            va,
                            SINGLE_OPCODE_TYPE,
                            mnem,
                            [ Msp430Operands[ds_addr_mode](dsreg, opData, dsopsize, va) ],
                            flags,
                            opData.lenData()
                            )
                    # FILTHY HACK
                    for oper in op.opers:
                        oper._dis_regctx = self._dis_regctx
                    return op
        # Primary Functional registers index values
        #REG_PC = 0  # reg0 is the Program Counter
        #REG_SP = 1  # reg1 is the Stack Pointer
        #REG_SR = 2  # reg2 is the Status Register
        #REG_CG = 3  # reg3 is the Constant Generator

    def getPointerSize(self):
        return 2

    def getProgramCounterIndex(self):
        return REG_PC

    def getStackCounterIndex(self):
        return REG_SP

    def getFrameCounterIndex(self):
        return None
            
    def getRegisterCount(self):
        """
        Return the number of registers
        """
        return len(registers)

    def clearRegisters(self, regcount):
        array = []
        for i in range(regcount):
            array.append(0)
        return array


Msp430Operands = [
        Msp430RegDirectOper,
        Msp430RegIndexOper,
        Msp430RegIndirOper,
        Msp430RegIndirAutoincOper,
        Msp430JmpOper,
]



"""
The vivisect.parsers package contains all the known file format parsers
for vivisect.  Each parser module must implement the following functions:

    parseFile(workspace, filename):
        Load the file into the given workspace
    parseBytes(workspace, bytes):
        Load the file (pre-read in) into the workspace

"""
# Some parser utilities

import io
import sys
import struct
import hashlib
import zipfile

import vstruct.defs.macho as vs_macho

def md5File(filename):
    d = hashlib.md5()
    with open(filename, 'rb') as f:
        bytes = f.read(4096)
        while len(bytes):
            d.update(bytes)
            bytes = f.read(4096)
    return d.hexdigest()

def md5Bytes(bytes):
    d = hashlib.md5()
    d.update(bytes)
    return d.hexdigest()

def sha256File(filename):
    with open(filename, 'rb') as f:
        return hashlib.sha256(f.read()).hexdigest().upper()

def sha256Bytes(bytes):
    return hashlib.sha256(bytes).hexdigest().upper()

macho_magics = (
    vs_macho.MH_MAGIC,
    vs_macho.MH_CIGAM,
    vs_macho.MH_MAGIC_64,
    vs_macho.MH_CIGAM_64,
    vs_macho.FAT_MAGIC,
    vs_macho.FAT_CIGAM,
)

def guessFormat(bytez):
    if bytez.startswith(b'VIV'):
        return 'viv'

    if b'MSGVIV' in bytez[:8]:
        return 'mpviv'

    if bytez.startswith(b"MZ"):
        return 'pe'

    if bytez.startswith(b'\x7fELF'):
        return 'elf'

    if bytez.startswith(b'\x7fCGC'):
        return 'cgc'

    bytemagic = struct.unpack('<I', bytez[:4])[0]
    if bytemagic in macho_magics:
        return 'macho'

    if bytez[0] == ord(':'):
        return 'ihex'

    if bytez[0] == ord('S'):
        return 'srec'

    return 'blob'


def guessFormatFilename(filename):
    with open(filename, 'rb') as f:
        return guessFormat(f.read(32))


def getParserModule(fmt):
    mname = "vivisect.parsers.%s" % fmt
    mod = sys.modules.get(mname)
    if mod is None:
        __import__(mname)
        mod = sys.modules[mname]
    return mod


def getBytesParser(fmt):
    if fmt == 'pe':
        import PE
        return PE.peFromBytes
    elif fmt == 'elf':
        import Elf
        return Elf.elfFromBytes
    return None


ZIPKEY = 'filebytes'


def compressBytes(byts):
    with io.BytesIO() as fd:
        with zipfile.ZipFile(fd, mode='w', compression=zipfile.ZIP_DEFLATED) as zipr:
            zipr.writestr(ZIPKEY, byts)
        fd.seek(0)
        return fd.read()


def uncompressBytes(byts):
    with io.BytesIO(byts) as fd:
        with zipfile.ZipFile(fd, mode='r', compression=zipfile.ZIP_DEFLATED) as zipr:
            return zipr.read(ZIPKEY)



'''
Register definition for the z80 architecture
'''

import envi.registers as e_reg

z80regs = [
    ('AF', 16),
    ('BC', 16),
    ('DE', 16),
    ('HL', 16),

    ('IX', 16),
    ('IY', 16),

    ('PC', 16),
    ('SP', 16),

    ('I', 8),
    ('R', 8),
]

l = locals()
e_reg.addLocalEnums(l, z80regs)

z80meta = [
    ('A', REG_AF, 8, 8),
    ('B', REG_BC, 8, 8),
    ('C', REG_BC, 0, 8),
    ('D', REG_DE, 8, 8),
    ('E', REG_DE, 0, 8),
    ('F', REG_AF, 0, 8),
    ('H', REG_HL, 8, 8),
    ('L', REG_HL, 0, 8),
]

e_reg.addLocalMetas(l, z80meta)

class z80RegisterContext(e_reg.RegisterContext):
    def __init__(self):
        e_reg.RegisterContext.__init__(self)
        self.loadRegDef(z80regs)
        self.loadRegMetas(z80meta)
        self.setRegisterIndexes(REG_PC, REG_SP)

regctx = z80RegisterContext()




from PyQt5.QtWidgets import *

from vqt.main import getSaveFileName

class MemDumpDialog(QDialog):
    '''
    gui for memdump cli command.
    '''
    def __init__(self, va, filename='', size=256):
        QDialog.__init__(self)

        self.va = va
        self.filename = filename
        self.size = size

        hbox1 = QHBoxLayout()
        fname_label = QLabel('Filename: ')
        self.fname_edit = QLineEdit()
        self.fname_edit.setText(self.filename)
        browse_button = QPushButton('...')
        browse_button.clicked.connect(self.showSaveAsDialog)
        hbox1.addWidget(fname_label)
        hbox1.addWidget(self.fname_edit)
        hbox1.addWidget(browse_button)

        hbox2 = QHBoxLayout()
        size_label = QLabel('Size: ')
        self.size_edit = QLineEdit()
        self.size_edit.setText(str(self.size))
        hbox2.addWidget(size_label)
        hbox2.addWidget(self.size_edit)

        hbox3 = QHBoxLayout()
        ok_button = QPushButton('&OK')
        ok_button.setDefault(True)
        ok_button.clicked.connect(self.okClicked)
        cancel_button = QPushButton('&Cancel')
        cancel_button.clicked.connect(self.cancelClicked)
        hbox3.addWidget(ok_button)
        hbox3.addWidget(cancel_button)

        vbox = QVBoxLayout()
        vbox.addLayout(hbox1)
        vbox.addLayout(hbox2)
        vbox.addLayout(hbox3)

        self.setLayout(vbox)

        title = 'Dump memory at va 0x%x to a file' % self.va
        self.setWindowTitle(title)

    def showSaveAsDialog(self):
        fname = str(getSaveFileName(caption='Select file to dump memory to'))
        self.fname_edit.setText(fname)

    def cancelClicked(self):
        self.close()

    def okClicked(self):
        self.filename = self.fname_edit.text()
        self.size = int(self.size_edit.text())

        self.accept()
        self.close()

    def getResults(self):
        return self.filename, self.size



import os
import bs4
import pickle
import requests
import tempfile
'''
Downloads lots of pages, parses, and pulls out function declarations for impapi

(210409)
One problem: 7 functions from "Waiting with Specific Clocks" page don't have 
all their arguments fit in the <dt> tags.  Rather than rework the script that
works for 1500+ other functions, we just modified the output manually.
(https://www.gnu.org/software/libc/manual/html_node/Waiting-with-Explicit-Clocks.html)


'''

BASE_URL = "https://www.gnu.org/software/libc/manual/html_node/"
def getAllFrontUrls(sess):
    frontpage = sess.get(BASE_URL + 'Function-Index.html') 
    soup = bs4.BeautifulSoup(frontpage.text)
    urls = []
    for link in soup.find_all('a'):
        url = link.get('href') 
        uurl = url 
        if '#' in uurl: 
            uurl = url[:url.find('#')] 
 
        if url in urls:
            continue

        urls.append(uurl)

    return urls


def getAllFuncProtos(funcs=None):
    if funcs is None:
        funcs = getRawDataLines()
    api = parseDataLines(funcs)

    return api


def getRawDataLines():
    with requests.session() as sess:

        # get all 1500+ urls and downselect them to functions 
        urls = getAllFrontUrls(sess)
        pages = [sess.get(BASE_URL + url) for url in urls]

        bss = [bs4.BeautifulSoup(page.text) for page in pages]

        dts = []
        funcdts = []
        for b in bss:
            dt = b.find_all('dt')

            for item in dt:
                if 'Function:' in repr(item):
                    if item not in funcdts:
                        funcdts.append(item)

        # strip out the text and whittle each function down to a prototype line
        functext = [fdt.text for fdt in funcdts]
        funcs = [line[10:] for line in functext]  # skip first 10 chars, "Function: "
        return funcs

def parseDataLines(funcs):
    # parse each line and populate the api dict
    api = {}
    for func in funcs:
        func = func.replace("*", "* ").replace(';', '')
        print(func)
        # split at the (
        leftside, rightside = func.split('(', 1)

        # handle the left side
        parts = [x for x in leftside.split(' ') if len(x)]

        funcname = parts.pop()
        rettype = ' '.join(parts)

        # handle the args (rightside)
        rightside = rightside.replace(')','')
        args = rightside.split(',')
        argchunks = [arg.split(' ') for arg in args]

        argout = []
        if argchunks != [['void']]:     # skip "(void)" entries
            for chunk in argchunks:
                chunk = [foo.strip() for foo in chunk if len(foo.strip())]
                if len(chunk):
                    if chunk[-1].endswith(')') or len(chunk) == 1:
                        aname = None
                    else:
                        aname = chunk.pop()

                    # handle variadic args (...):
                    if len(chunk) and chunk[-1] in ('...', b'\xe2\x80\xa6'.decode('utf8')):
                        atype = 'variadic'
                        aname = ''
                    else:
                        atype = ' '.join(chunk)

                    argout.append((atype, aname))
                else:
                    input("FIXME: lost args at the end")

        if 'printf' in func:
            #import envi.interactive as ei; ei.dbg_interact(locals(), globals())
            pass


        cconv = cconvs.get(funcname, 'cdecl')
        
        api['plt_%s' % funcname] = (rettype, None, cconv, '*.%s' % funcname, tuple(argout))

    return api

cconvs = {
        'div': 'stdcall',
        'ldiv': 'stdcall',
        'lldiv': 'stdcall',
        'mallinfo': 'stdcall',
        'inet_makeaddr': 'stdcall',
        }


def reprAllFuncProtos(funcs=None):
    api = getAllFuncProtos(funcs)
    keys = list(api.keys())
    keys.sort()

    out = ['apitypes = {\n}','','api = {']
    out.append("    'plt___libc_start_main':( 'int', None, 'cdecl', '*.__libc_start_main', (('int', 'main'), ('int', 'argc'), ('int', 'argv')) ),")
    out.append("    'main':( 'int', None, 'stdcall', '*.main_entry', (('int', None), ('int', None), ('int', None)) ),")

    for key in keys:
        data = api.get(key)
        out.append("    %r: %r," % (key, data)
                )
    out.append('''    # taken from libc directly:
    'plt___libc_rpc_getport': ('int', None, 'stdcall', '*.__libc_rpc_getport', (('int', 'arg0'), ('int', 'arg1'), ('int', 'arg2'))),
    'plt___nss_services_lookup2': ('int', None, 'stdcall', '*.__nss_services_lookup2', (('int', 'arg0'),)),
    'plt___nss_passwd_lookup2': ('int', None, 'stdcall', '*.__nss_passwd_lookup2', (('int', 'arg0'),)),
    'plt__dl_addr': ('int', None, 'stdcall', '*._dl_addr', (('int', 'arg0'),)),
    'plt__dl_vsym': ('int', None, 'stdcall', None, (('int', 'arg0'),)),
    'plt___nss_group_lookup2': ('int', None, 'stdcall', '*.__nss_group_lookup2', (('int', 'arg0'),)),
    'plt___nss_hosts_lookup2': ('int', None, 'stdcall', '*.__nss_hosts_lookup2', (('int', 'arg0'),)),
    ''')
    out.append('}')
    return '\n'.join(out) + "\n"

def writeAllFuncProtos():
    open('i386.py', 'w').write(reprAllFuncProtos())


if __name__ == '__main__':
    writeAllFuncProtos()



"""
Some utilities related to i386 analysis.  Loaders and analysis
modules may use these as needed...
"""

import binascii

sigs = [
    ("558bec", "ffffff"),  # push ebp; mov ebp,esp; Intel/Microsoft
    ("568bf1", "ffffff"),  # push esi; mov esi,ecx (c++)
    ("5589e5", "ffffff"),  # push ebp; mov ebp,esp; GCC
    ("8bff558bec", "ffffffffff"),  # mov edi,edi; push ebp; mov epb, esp
    # Ok... here's where things get cool...
    # This is push <imm8>, push <imm32>, call <somewhere> # ms seh setup entry
    ("6a006800000000e8", "ff00ff00000000ff")
]


def addEntrySigs(vw):
    for sigstr, maskstr in sigs:
        bytez = binascii.unhexlify(sigstr)
        masks = binascii.unhexlify(maskstr)
        vw.addFunctionSignatureBytes(bytez, masks)



'''
Visgraph supports backing the graph objects with a postgres db.
'''

import psycopg2
import collections
import visgraph.graphcore as vg_graphcore

init_db = '''
    DROP TABLE IF EXISTS vg_edges;
    CREATE TABLE vg_edges (
        eid     BIGSERIAL,
        n1      BIGINT,
        n2      BIGINT,
        created TIMESTAMP DEFAULT NOW(),
        PRIMARY KEY (eid)
    );
    CREATE INDEX vg_edges_idx_n1 ON vg_edges (n1);
    CREATE INDEX vg_edges_idx_n2 ON vg_edges (n2);

    DROP TABLE IF EXISTS vg_edge_props;
    CREATE TABLE vg_edge_props (
        eid     BIGINT,
        pname   VARCHAR(256) NOT NULL,
        intval  BIGINT,
        strval  VARCHAR(1024),
        created TIMESTAMP DEFAULT NOW(),
        PRIMARY KEY (eid, pname)
    );
    CREATE INDEX vg_edge_eid_idx ON vg_edge_props (eid);
    CREATE INDEX vg_edge_pname_intval ON vg_edge_props (pname, intval);
    CREATE INDEX vg_edge_pname_strval ON vg_edge_props (pname, strval);

    DROP TABLE IF EXISTS vg_nodes;
    CREATE TABLE vg_nodes (
        nid     BIGSERIAL,
        created TIMESTAMP DEFAULT NOW(),
        PRIMARY KEY(nid)
    );

    DROP TABLE IF EXISTS vg_node_props;
    CREATE TABLE vg_node_props (
        nid     BIGINT NOT NULL,
        pname   VARCHAR(255) NOT NULL,
        intval  BIGINT,
        strval  VARCHAR(1024),
        created TIMESTAMP DEFAULT NOW(),
        PRIMARY KEY  (nid,pname)
    );
    CREATE INDEX vg_node_nid_idx ON vg_node_props (nid);
    CREATE INDEX vg_node_pname_intval ON vg_node_props (pname, intval);
    CREATE INDEX vg_node_pname_strval ON vg_node_props (pname, strval);
'''

# Exanmple database creds...
default_dbinfo = {
'user':'visgraph',
'password':'ohhai!',
'database':'visgraph',
# Add host if you want...
}

def initGraphDb(dbinfo):
    db = psycopg2.connect(**dbinfo)
    c = db.cursor()
    c.execute(init_db)
    c.close()
    db.commit()
    db.close()

# Rollback transactions on exception
def rollsafe(f):
    def doroll(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except Exception as e:
            try:
                args[0].db.rollback()
            except Exception:
                pass
            raise

    doroll.__doc__ == f.__doc__
    doroll.__name__ == f.__name__
    return doroll

class DbGraphStore:
    '''
    A DbGraphStore object may be used for all the standard management
    of node and edge information but may not be used for path queries.

    FIXME possibly make it able to do path queries but *really* slow?

    Use the buildSubGraph() API to pull path serchable graphs out of
    the DBGraphStore.
    '''
    def __init__(self, dbinfo=None):

        if dbinfo == False:
            return

        if dbinfo is None:
            dbinfo = default_dbinfo

        self.dbinfo = dbinfo
        self.db = psycopg2.connect(**dbinfo)
        self.autocommit = True

    @rollsafe
    def _doSelect(self, query, *args):
        '''
        For now, a fetchall based select wrapper.
        '''
        c = self.db.cursor()
        c.execute(query, args)
        res = c.fetchall()
        c.close()
        if self.autocommit:
            self.db.commit()
        return res

    @rollsafe
    def _doInsert(self, query, *args):
        '''
        Standard insert wrapper.
        '''
        c = self.db.cursor()
        c.execute(query, args)
        c.close()
        if self.autocommit:
            self.db.commit()

    @rollsafe
    def _doUpdate(self, query, *args):
        '''
        Do an update with 'returning' syntax to know
        if an update was made.
        '''
        res = []
        c = self.db.cursor()
        c.execute(query, args)
        res = c.fetchall()
        c.close()
        if self.autocommit:
            self.db.commit()
        return res

    @rollsafe
    def _doInsertRetId(self, query, *args):
        '''
        Insert with a returning value.
        '''
        c = self.db.cursor()
        c.execute(query, args)
        hid = c.fetchall()[0][0]
        c.close()
        if self.autocommit:
            self.db.commit()
        return hid

    @rollsafe
    def _doInsertRetIds(self, query, *args):
        '''
        Insert with a returning list of IDs.
        '''
        c = self.db.cursor()
        c.execute(query, args)
        rows = c.fetchall()
        c.close()
        if self.autocommit:
            self.db.commit()
        return [ r[0] for r in rows ]

    def _doCommit(self):
        self.db.commit()

    def addNode(self, nodeid=None, ninfo=None, **kwargs):
        if nodeid is not None:
            raise Exception('DbGraphStore Manages nodeid!')
        q = 'INSERT INTO vg_nodes DEFAULT VALUES RETURNING nid'
        nid = self._doInsertRetId(q)

        if ninfo is not None:
            kwargs.update(ninfo)

        for key,val in kwargs.items():
            self.setNodeProp(nid, key, val)

        return nid

    def delEdge(self, eid):
        '''
        Delete an edge from the graph database.

        Example: g.delEdge(eid)
        '''
        q = '''
        DELETE FROM
            vg_edge_props
        WHERE
            eid = %s
        '''
        self._doInsert(q, eid)
        q = '''
        DELETE FROM
            vg_edges
        WHERE
            eid = %s
        '''
        self._doInsert(q, eid)

    def delNode(self, nid):
        '''
        Delete the given node (and his edges) from the graph dbase.

        Example: g.delNode(nid)

        NOTE: this will delete any edges which go to or from nid!
        '''

        # Delete edge properties to and from
        q = '''
        DELETE FROM
            vg_edge_props
        USING
            vg_edges
        WHERE 
            vg_edges.n1 = %s
          AND
            vg_edges.eid = vg_edge_props.eid
        '''
        self._doInsert(q, nid)

        q = '''
        DELETE FROM
            vg_edge_props
        USING
            vg_edges
        WHERE 
            vg_edges.n2 = %s
          AND
            vg_edges.eid = vg_edge_props.eid
        '''
        self._doInsert(q, nid)

        # Delete edges to and from
        q = '''
        DELETE FROM
            vg_edges
        WHERE
            vg_edges.n1 = %s
        '''
        self._doInsert(q, nid)

        q = '''
        DELETE FROM
            vg_edges
        WHERE
            vg_edges.n2 = %s
        '''
        self._doInsert(q, nid)

        # Delete from node properties
        q = '''
        DELETE FROM
            vg_node_props
        WHERE
            nid = %s
        '''
        self._doInsert(q, nid)
        q = '''
        DELETE FROM
            vg_nodes
        WHERE
            nid = %s
        '''
        self._doInsert(q, nid)

    def setNodeProp(self, nid, pname, value):

        if isinstance(value, bool):
            value = int(value)

        if isinstance(value, int):
            q = 'UPDATE vg_node_props SET intval=%s,created=NOW() WHERE nid=%s and pname=%s RETURNING nid'
            q1 = 'INSERT INTO vg_node_props (nid, pname, intval) VALUES (%s,%s,%s)'
        else:
            q = 'UPDATE vg_node_props SET strval=%s,created=NOW() WHERE nid=%s and pname=%s RETURNING nid'
            q1 = 'INSERT INTO vg_node_props (nid, pname, strval) VALUES (%s,%s,%s)'

        # return a value to see if we actually did the update...
        res = self._doSelect(q, value, nid, pname)
        if len(res) == 0:
            self._doInsert(q1, nid, pname, value)
        if self.autocommit:
            self.db.commit()

    def getNodeProp(self, nid, pname, default=None):
        q = 'SELECT intval,strval from vg_node_props WHERE nid=%s AND pname=%s'
        res = self._doSelect(q, nid, pname)
        if len(res) == 0:
            return default
        intval, strval = res[0]
        if intval is not None:
            return intval
        return strval

    def delNodeProp(self, nid, pname):
        q = 'DELETE FROM vg_node_props WHERE nid=%s AND pname=%s'
        self._doInsert(q, nid, pname)

    def getNodeProps(self, nid):
        ret = {}
        q = 'SELECT pname,intval,strval FROM vg_node_props WHERE nid=%s'
        for pname,intval,strval in self._doSelect(q, nid):
            if intval is not None:
                ret[pname] = intval
            else:
                ret[pname] = strval
        return ret

    def getNodesProps(self, nids):
        ret = collections.defaultdict(dict)
        q = 'SELECT nid,pname,intval,strval FROM vg_node_props WHERE nid IN %s' 
        for nid,pname,intval,strval in self._doSelect(q, tuple(nids)):
            if intval is not None:
                ret[nid][pname] = intval
            else:
                ret[nid][pname] = strval
        return ret.items()

    def addEdge(self, fromid, toid, eid=None, einfo=None):
        if eid is not None:
            raise Exception('DbGraphStore Manages eid!')
        if fromid is None:
            raise Exception('Invalid from id (None)!')
        if toid is None:
            raise Exception('Invalid to id (None)!')
        q = 'INSERT INTO vg_edges (n1, n2) VALUES (%s, %s) RETURNING eid'
        eid = self._doInsertRetId(q, fromid, toid)
        if einfo is not None:
            for key,val in einfo.items():
                self.setEdgeProp(eid, key, val)
        return eid

    def getRefsFrom(self, nodeid):
        '''
        Return a list of edges which originate with us.

        Example: for eid, fromid, toid, einfo in g.getRefsFrom(id)
        '''
        q = '''
        SELECT
            vg_edges.*,
            vg_edge_props.*
        FROM
            vg_edges,
            vg_edge_props
        WHERE
            vg_edges.n1 = %s AND
            vg_edges.eid = vg_edge_props.eid
        '''
        refs = {}
        res = self._doSelect(q, nodeid)
        for eid,n1,n2,created,eid1,pname,intval,strval,created1 in res:
            r = refs.get(eid)
            if r is None:
                r = (eid, n1, n2, {})
                refs[eid] = r

            if intval is not None:
                r[3][pname] = intval
            else:
                r[3][pname] = strval

        return refs.values()

    def getRefsTo(self, nodeid):
        '''
        Return a list of edges which we reference.

        Example: for eid, fromid, toid, einfo in g.getRefsTo(id)
        '''
        q = '''
        SELECT
            vg_edges.*,
            vg_edge_props.*
        FROM
            vg_edges,
            vg_edge_props
        WHERE
            vg_edges.n2 = %s AND
            vg_edges.eid = vg_edge_props.eid
        '''
        refs = {}
        res = self._doSelect(q, nodeid)
        for eid,n1,n2,created,eid1,pname,intval,strval,created1 in res:
            r = refs.get(eid)
            if r is None:
                r = (eid, n1, n2, {})
                refs[eid] = r

            if intval is not None:
                r[3][pname] = intval
            else:
                r[3][pname] = strval

        return refs.values()
    
    def getRefsFromBulk(self, nids):
        '''
        Return a list of edges which originate with us.
        Supply a list of edges to get refs.

        Example: for eid, fromid, toid, einfo in g.getRefsFromBulk(nids)
        '''
        q = '''
        SELECT
            vg_edges.eid, vg_edges.n1, vg_edges.n2,
            vg_edge_props.pname, vg_edge_props.intval, vg_edge_props.strval
        FROM
            vg_edges,
            vg_edge_props
        WHERE
            vg_edges.n1 IN (%s) AND
            vg_edges.eid = vg_edge_props.eid
        '''
        if not nids:
            return []
        refs = {}
        qend = ','.join( ['%s',] * len(nids))
        q = q % qend
        res = self._doSelect(q, *nids)
        for eid, n1, n2, pname, intval, strval in res:
            r = refs.get(eid)
            if r is None:
                r = (eid, n1, n2, {})
                refs[eid] = r

            if intval is not None:
                r[3][pname] = intval
            else:
                r[3][pname] = strval

        return list(refs.values())

    def getRefsToBulk(self, nids):
        '''
        Return a list of edges which we reference.
        Supply a list of edges to gets refs.

        Example: for eid, fromid, toid, einfo in g.getRefsToBulk(nids)
        '''
        q = '''
        SELECT
            vg_edges.eid, vg_edges.n1, vg_edges.n2,
            vg_edge_props.pname, vg_edge_props.intval, vg_edge_props.strval
        FROM
            vg_edges,
            vg_edge_props
        WHERE
            vg_edges.n2 IN (%s) AND
            vg_edges.eid = vg_edge_props.eid
        '''
        if not nids:
            return []
        refs = {}
        qend = ','.join( ['%s',] * len(nids))
        q = q % qend
        res = self._doSelect(q, *nids)
        for eid, n1, n2, pname, intval, strval in res:
            r = refs.get(eid)
            if r is None:
                r = (eid, n1, n2, {})
                refs[eid] = r

            if intval is not None:
                r[3][pname] = intval
            else:
                r[3][pname] = strval

        return list(refs.values())

    def setEdgeProp(self, eid, pname, value):

        if isinstance(value, bool):
            value = int(value)

        if isinstance(value, int):
            q = 'UPDATE vg_edge_props SET intval=%s WHERE eid=%s and pname=%s RETURNING eid'
            q1 = 'INSERT INTO vg_edge_props (eid, pname, intval) VALUES (%s,%s,%s)'
        else:
            q = 'UPDATE vg_edge_props SET strval=%s WHERE eid=%s and pname=%s RETURNING eid'
            q1 = 'INSERT INTO vg_edge_props (eid, pname, strval) VALUES (%s,%s,%s)'
        # return a value to see if we actually did the update...
        res = self._doSelect(q, value, eid, pname)
        if len(res) == 0:
            self._doInsert(q1, eid, pname, value)

    def getEdgeProp(self, eid, pname, default=None):
        q = 'SELECT intval,strval from vg_edge_props WHERE eid=%s AND pname=%s'
        res = self._doSelect(q, eid, pname)
        if len(res) == 0:
            return default
        intval, strval = res[0]
        if intval is not None:
            return intval
        return strval

    def getEdge(self, eid):
        '''
        Get the edge tuple ( eid, n1, n2, nprops ) for the given edge by id.
        '''
        q = 'SELECT eid,n1,n2 FROM vg_edges WHERE eid=%s'
        res = self._doSelect( q, eid )
        if not res:
            raise Exception('Invalid Edge Id: %s' % eid)
        e,n1,n2 = res[0]
        return (eid, n1, n2, self.getEdgeProps( eid ) )

    def getEdgeProps(self, eid):
        '''
        Retrieve the properties dictionary for the given edge id.
        '''
        ret = {}
        q = 'SELECT pname,intval,strval FROM vg_edge_props WHERE eid=%s'
        for pname,intval,strval in self._doSelect(q, eid):
            if intval is not None:
                ret[pname] = intval
            else:
                ret[pname] = strval
        return ret

    def searchNodes(self, propname, propval=None):
        '''
        Return (but do not cache forward) the nid's of nodes which
        have a property with the following name (and optionally, value).

        Example:
            for nid in g.searchNodes('woot', 10)
                print(g.getNodeProp(nid, 'name'))

        NOTE: This is specific to the DbGraphStore...
        '''
        if propval is None:
            q = 'SELECT nid FROM vg_node_props WHERE pname=%s'
            c = self.db.cursor()
            c.execute(q, (propname,))
            for row in c:
                yield row
            c.close()

    def buildSubGraph(self):
        '''
        Return a subgraph which may be used to populate from the DB and
        do path searching.
        '''
        return DbSubGraph(self.dbinfo)

class DbSubGraph(DbGraphStore, vg_graphcore.Graph):

    '''
    A subgraph in the database is basically a forward cached instance of selected
    nodes and edges in an in-memory graph (visgraph.graphcore.Graph).  This object
    may then be used for traditional path tracing without going back to the database.

    Any modifications to graph element properties *will* be synchronized back to the
    database backing the given subgraph.
    '''

    def __init__(self, dbinfo):
        vg_graphcore.Graph.__init__(self)
        DbGraphStore.__init__(self, dbinfo)

    def addNode(self, nodeid=None, ninfo=None, **kwargs):
        # Do *both*
        nid = DbGraphStore.addNode(self, nodeid=nodeid, ninfo=ninfo, **kwargs)
        vg_graphcore.Graph.addNode(self, nodeid=nid, ninfo=None, **kwargs)
        return nid

    def addEdge(self, fromid, toid, einfo):
        eid = DbGraphStore.addEdge(self, fromid, toid, einfo=einfo)
        vg_graphcore.Graph.addEdge(self, fromid, toid, eid=eid, einfo=None)
        return eid

    def useEdges(self, **kwargs):
        '''
        Pull some edges from the DbStore backing this subgraph into the actual
        visgraph.graphcore.Graph instance so path traversal is possible.
        '''
        done = {}
        for key,val in kwargs.items():
            if isinstance(val, int):
                # FIXME is vg_edges.eid faster or vg_edge_props?
                q = 'SELECT vg_edges.eid,n1,n2 FROM vg_edge_props,vg_edges WHERE pname=%s AND intval=%s AND vg_edges.eid=vg_edge_props.eid'
            else:
                q = 'SELECT vg_edges.eid,n1,n2 FROM vg_edge_props,vg_edges WHERE pname=%s AND strval=%s AND vg_edges.eid=vg_edge_props.eid'
            for eid,n1,n2 in self._doSelect(q, key, val):
                done[eid] = (eid, n1, n2)

        # FIXME add the nodes for these edges
        for eid, n1, n2 in done.values():
            if vg_graphcore.Graph.getNode(self, n1) is None:
                vg_graphcore.Graph.addNode(self, nodeid=n1)
            if vg_graphcore.Graph.getNode(self, n2) is None:
                vg_graphcore.Graph.addNode(self, nodeid=n2)
            vg_graphcore.Graph.addEdge(self, n1, n2, eid=eid)

    def expandNode(self, nid, maxdepth=1):
        '''
        Add *all* the edges (and adjacent nodes) by traversing this nodes
        edges to the specified depth...
        '''
        todo = [(nid, 0),]
        if vg_graphcore.Graph.getNode(self, nid) is None:
            vg_graphcore.Graph.addNode(self, nodeid=nid)

        while len(todo):
            nid,depth = todo.pop()

            if depth > maxdepth:
                continue

            # Do expansion based on the *database*
            q = 'SELECT eid,n2 FROM vg_edges WHERE n1=%s'
            for eid, n2 in self._doSelect(q, nid):
                if vg_graphcore.Graph.getNode(self, n2) is None:
                    vg_graphcore.Graph.addNode(self, nodeid=n2)
                if vg_graphcore.Graph.getEdge(self, eid) is None:
                    vg_graphcore.Graph.addEdge(self, nid, n2, eid=eid)
                ndepth = depth+1
                if ndepth < maxdepth:
                    todo.append((n2, ndepth))

    # pullNode?
    # expandNode?

